from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.item_image_refs import Item_image_refs, Item_image_refsManager


@JsonResponseWithException()
def Item_image_refs_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Item_image_refs.objects.
                filter(thumb__isnull=False).
                get_range_rows1(
                request=request,
                function=Item_image_refsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_image_refs_Add(request):
    return JsonResponse(DSResponseAdd(data=Item_image_refs.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_image_refs_Update(request):
    return JsonResponse(DSResponseUpdate(data=Item_image_refs.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_image_refs_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Item_image_refs.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_image_refs_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Item_image_refs.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_image_refs_Info(request):
    return JsonResponse(DSResponse(request=request, data=Item_image_refs.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_image_refs_Copy(request):
    _request = DSRequest(request=request)

    source = _request.json.get('source')
    mode = source.get('mode')
    srecords = source.get('records')

    destination = _request.json.get('destination')
    drecords = destination.get('records')

    res = False
    if isinstance(srecords, list):
        for srecord in srecords:
            for drecord in drecords:
                if drecord.get('child_id'):
                    item_refs, _ = Item_image_refs.objects.get_or_create(item_id=drecord.get('child_id'), thumb_id=srecord.get('id'))
                else:
                    item_refs, _ = Item_image_refs.objects.get_or_create(item_id=drecord.get('id'), thumb_id=srecord.get('id'))
                res = True

    if not res:
        raise Exception('Копирование не выполнено.')

    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.2 on 2020-01-27 07:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0124_auto_20200123_0314'),
    ]

    operations = [
        migrations.AddField(
            model_name='lotsman_documents_hierarcy',
            name='path',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.urls import path

from kaf_pas.sales.views import status_dogovor

urlpatterns = [

    path('StatusDogovor/Fetch/', status_dogovor.StatusDogovor_Fetch),
    path('StatusDogovor/Add', status_dogovor.StatusDogovor_Add),
    path('StatusDogovor/Update', status_dogovor.StatusDogovor_Update),
    path('StatusDogovor/Remove', status_dogovor.StatusDogovor_Remove),
    path('StatusDogovor/Lookup/', status_dogovor.StatusDogovor_Lookup),
    path('StatusDogovor/Info/', status_dogovor.StatusDogovor_Info),
    path('StatusDogovor/Copy', status_dogovor.StatusDogovor_Copy),

]



from django.urls import path

from kaf_pas.planing.views import production_order_values

urlpatterns = [

    path('Production_order_values/Fetch/', production_order_values.Production_order_values_Fetch),
    path('Production_order_values/Add', production_order_values.Production_order_values_Add),
    path('Production_order_values/Update', production_order_values.Production_order_values_Update),
    path('Production_order_values/Remove', production_order_values.Production_order_values_Remove),
    path('Production_order_values/Lookup/', production_order_values.Production_order_values_Lookup),
    path('Production_order_values/Info/', production_order_values.Production_order_values_Info),
    path('Production_order_values/Copy', production_order_values.Production_order_values_Copy),

]



# Generated by Django 2.2.8 on 2019-12-06 21:52

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0154_auto_20191206_1937'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='launch_operations_item',
            name='disabled_launch',
        ),
        migrations.AlterField(
            model_name='launch_operations_item',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'),), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-13 11:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0013_auto_20191013_1141'),
    ]

    operations = [
        migrations.RenameField(
            model_name='user_add_info',
            old_name='ip',
            new_name='description',
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.status_demand import Status_demand, Status_demandManager


@JsonResponseWithException()
def Status_demand_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Status_demand.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Status_demandManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_demand_Add(request):
    return JsonResponse(DSResponseAdd(data=Status_demand.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_demand_Update(request):
    return JsonResponse(DSResponseUpdate(data=Status_demand.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_demand_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Status_demand.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_demand_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Status_demand.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_demand_Info(request):
    return JsonResponse(DSResponse(request=request, data=Status_demand.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_demand_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Status_demand.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.5 on 2019-09-18 14:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0054_delete_demand'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='precent',
            unique_together=set(),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-08 14:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0201_auto_20200208_1133'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item_image_refs',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='item_image_refs',
            constraint=models.UniqueConstraint(condition=models.Q(('thumb10', None), ('thumb', None)), fields=('item',), name='Item_image_refs_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='item_image_refs',
            constraint=models.UniqueConstraint(condition=models.Q(thumb10=None), fields=('item', 'thumb'), name='Item_image_refs_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='item_image_refs',
            constraint=models.UniqueConstraint(condition=models.Q(thumb=None), fields=('item', 'thumb10'), name='Item_image_refs_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='item_image_refs',
            constraint=models.UniqueConstraint(fields=('item', 'thumb', 'thumb10'), name='Item_image_refs_unique_constraint_3'),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.system.models.contants import ContantsManager, Contants


@JsonResponseWithException()
def Contants_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Contants.objects.
                filter().
                get_range_rows1(
                request=request,
                function=ContantsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Contants_Add(request):
    return JsonResponse(DSResponseAdd(data=Contants.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Contants_Update(request):
    return JsonResponse(DSResponseUpdate(data=Contants.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Contants_RefreshMatView(request):
    return JsonResponse(DSResponseUpdate(data=Contants.objects.refreshMatViewFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Contants_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Contants.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)\

@JsonResponseWithException()
def Contants_fixed_num_in_operations(request):
    return JsonResponse(DSResponse(request=request, data=Contants.objects.fixed_num_in_operationsFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Contants_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Contants.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Contants_Info(request):
    return JsonResponse(DSResponse(request=request, data=Contants.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Contants_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Contants.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



import logging
from typing import List, Tuple

from django.core.management import BaseCommand
from django.db import transaction

from isc_common.progress import ProgressDroped
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
from kaf_pas.kd.models.lotsman_documents_hierarcy_refs import Lotsman_documents_hierarcy_refs
from kaf_pas.production.models import progress_deleted

logger = logging.getLogger(__name__)


def check_level(
        parent_array: List[Tuple[int, int, str]] = list(),
        item: Item or Lotsman_documents_hierarcy_refs = None,
        level: int = 0,
        table: str = '',
        cycle_ref: List = [],
        full_path: str = '',
        progress=None) -> List:
    res = 0

    if level > 50:
        raise Exception(f'Too big level {level}')

    # progress.setContentsLabel(f'in to level: {level}\n{item.id}\n')
    logger.debug(f'in to level: {level}\n{item.id}\n')

    if table == 'Item_refs':
        query = Item_refs.objects.filter(parent_id=item.id)

    elif table == 'Lotsman_documents_hierarcy_refs':
        query = Lotsman_documents_hierarcy_refs.objects.filter(parent_id=item.id)

    full_path = f'{full_path} / {item.item_name.rstrip()}'

    for item_refs in query:
        cycled = [item for item in parent_array if item[0] == item_refs.child.id and item[1] == item_refs.parent.id]

        if len(cycled) > 0:
            s = f'''<pre>\nlevel:{level}\nID PARENT: {item_refs.parent.id if item_refs.parent else 'None'} ({item_refs.parent.item_name})\nID CHILD: {item_refs.child.id} ({item_refs.child.item_name})</pre>'''
            logger.debug(s)
            cycle_ref.append(s)
            s = f'''<pre>Full path Parent:  {cycled[0][2]}</pre>'''
            cycle_ref.append(s)
            s = f'''<pre>Full path Child: {full_path}</pre>'''
            cycle_ref.append(s)
            # item_refs.delete()
        else:
            parent_array.append((item_refs.parent.id, item_refs.child.id, full_path))
            check_level(
                parent_array=parent_array,
                item=item_refs.child,
                level=level + 1,
                table=table,
                cycle_ref=cycle_ref, progress=progress,
                full_path=full_path
            )
        if progress:
            res = progress.step()

        if res != 0:
            raise ProgressDroped(progress_deleted)
        logger.debug(f'out to level: {level}\n')
    return cycle_ref


class Command(BaseCommand):
    help = "Нахождение циклических ссылок"

    def add_arguments(self, parser):
        parser.add_argument('--table', type=str)
        parser.add_argument('--item_id', type=int)

    def handle(self, *args, **options):
        table = options.get('table')
        item_id = options.get('item_id')

        print(f'table: {table}')
        print(f'item_id: {item_id}')

        cycle_ref = []

        parent_array = list()
        with transaction.atomic():
            if table == 'Item_refs':
                cycle_ref = check_level(parent_array=parent_array, item=Item.objects.get(id=item_id), level=1, table=table)
            elif table == 'Lotsman_documents_hierarcy_refs':
                cycle_ref = check_level(parent_array=parent_array, item=Lotsman_documents_hierarcy.objects.get(id=item_id), level=1, table=table)

        for item in cycle_ref:
            print(item)



import logging

from bitfield import BitField
from django.conf import settings
from django.db import transaction
from django.db.models import TextField, F

from isc_common import delAttr, setAttr, Stack
from isc_common.auth.models.user import User
from isc_common.bit import IsBitOn, TurnBitOn
from isc_common.common.mat_views import create_tmp_mat_view
from isc_common.datetime import DateToStr
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.number import DelProps
from isc_common.progress import managed_progress, ProgressDroped
from kaf_pas.ckk.models.item_line import Item_line
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.production.models import progress_deleted, p_id
from kaf_pas.production.models.operation_material import Operation_material
from kaf_pas.production.models.operation_resources import Operation_resources
from kaf_pas.production.models.operations_item import Operations_item
from kaf_pas.sales.models.demand import Demand

logger = logging.getLogger(__name__)


class Ready_2_launchQuerySet(AuditQuerySet):
    pass

    def get_info(self, request, *args):
        request = DSRequest(request=request)
        request = Ready_2_launchManager._del_options(request)
        criteria = self.get_criteria(json=request.json)
        cnt = super().filter(*args, criteria).count()
        cnt_all = super().filter().count()
        return dict(qty_rows=cnt, all_rows=cnt_all)

    def get_range_rows1(self, request, function=None, distinct_field_names=None):
        request = DSRequest(request=request)
        request = Ready_2_launchManager._del_options(request)
        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        res = self.get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json)
        return res


class Item_refs_Stack(Stack):
    def add_parents(self, id):

        for item_ref in Item_refs.objects.raw(f'''WITH RECURSIVE r AS (
                                                    SELECT *, 1 AS level
                                                    FROM ckk_item_refs
                                                    WHERE child_id IN (%s)
                                                    and parent_id != %s

                                                    union all

                                                    SELECT ckk_item_refs.*, r.level + 1 AS level
                                                    FROM ckk_item_refs
                                                             JOIN r
                                                                  ON ckk_item_refs.child_id = r.parent_id)

                                                select  *
                                                from r
                                                where child_id != %s order by level desc''', [
            id,
            p_id,
            p_id
        ]):
            self.push(item_ref)

    @property
    def _get_full_path_obj(self):
        arr = []
        last = self.top()
        if last:
            arr.append(last)
        while True:
            if last.parent != None:
                last = [item for item in self.stack if item.child.id == last.parent.id]
                if len(last) > 0:
                    last = last[0]
                    arr.append(last)
                else:
                    break
            else:
                # arr.append(last)
                break

        arr = [item for item in reversed(arr)]
        return arr

    @property
    def get_full_path_obj(self):
        from kaf_pas.ckk.models.item_operations_view import Item_operations_view
        from kaf_pas.ckk.models.item_operations_view import Item_operations_viewManager
        arr = [Item_operations_viewManager.getRecord(Item_operations_view.objects.get(refs_id=item.id)) for item in self._get_full_path_obj]
        return arr

    @property
    def get_full_path(self):
        arr = self._get_full_path_obj
        # arr = [item for item in arr if arr.parent != None]
        res = ' / '.join([item.child.item_name for item in arr])
        return '/ ' + res


class Ready_2_launchManager(AuditManager):

    @staticmethod
    def _del_options(request):
        delAttr(request.json.get('data'), 'full_name')
        delAttr(request.json.get('data'), 'check_qty')
        delAttr(request.json.get('data'), 'check_material')
        delAttr(request.json.get('data'), 'check_resources')
        delAttr(request.json.get('data'), 'check_edizm')
        delAttr(request.json.get('data'), 'check_operation')
        delAttr(request.json.get('data'), 'check_colvo')
        return request

    @staticmethod
    def make(demand, user, ready_2_launch=None, props=32):
        from kaf_pas.production.models.ready_2_launch_detail import Ready_2_launch_detail
        from isc_common.models.deleted_progresses import Deleted_progresses

        if isinstance(demand, int):
            demand = Demand.objects.get(id=demand)
        elif not isinstance(demand, Demand):
            raise Exception(f'demand must be a Demand instance or int')

        if isinstance(user, int):
            user = User.objects.get(id=user)
        elif not isinstance(user, User):
            raise Exception(f'user must be a User instance or int')

        cnt = 0
        cnt_not = 0
        step_res = 0

        all_notes = []
        options = []

        if IsBitOn(props, 0):
            options.append('Включена опция проверки наличия у операции длительности выполнения.')

        if IsBitOn(props, 1):
            options.append('Включена опция проверки наличия у операции № п/п.')

        if IsBitOn(props, 2):
            options.append('Включена опция проверки наличия у операции материалов или стандартных изделий.')

        if IsBitOn(props, 3):
            options.append('Включена опция проверки наличия у операции ресурса либо места выполнения.')

        if IsBitOn(props, 4):
            options.append('Включена опция проверки наличия у операции единицы измерения.')

        if IsBitOn(props, 5):
            options.append('Включена опция проверки наличия операций.')

        if IsBitOn(props, 6):
            options.append('Включена опция проверки количества.')

        demand_str = f'<h3>Оценка готовности к запуску: Заказ № {demand.code} от {DateToStr(demand.date)}</h3>'

        try:
            settings.LOCKS.acquire(f'Ready_2_launchManager.make_{demand.id}')
            with transaction.atomic():
                for _ in Demand.objects.filter(id=demand.id).select_for_update():
                    with managed_progress(
                            id=f'demand_{demand.id}_{ready_2_launch.props}_{user.id}',
                            qty=Item_refs.objects.get_descendants_count(
                                id=demand.precent_item.item.id,
                                # distinct='distinct',
                                where_clause=f'''where parent_id != {p_id}'''
                            ),
                            user=user,
                            message=demand_str,
                            title='Выполнено',
                            props=TurnBitOn(0, 0)

                    ) as progress:

                        if not ready_2_launch:
                            ready_2_launch, _ = Ready_2_launch.objects.get_or_create(demand=demand)

                        items_refs_stack = Item_refs_Stack()
                        items_refs_stack.add_parents(demand.precent_item.item.id)

                        for item_ref in Item_refs.objects.get_descendants(
                                id=demand.precent_item.item.id,
                                where_clause=f'''where parent_id != {p_id}  and is_bit_on(props::integer, 1) = true''',
                                # distinct='distinct'
                        ):
                            notes = []

                            # items_refs_stack.push(item_ref, lambda stack, item: len([it for it in stack if it.id != item_ref.id]) == 0)
                            items_refs_stack.push(item_ref)

                            operations_cnt = Operations_item.objects.filter(item=item_ref.child).count()
                            section = None
                            item_full_name = None
                            item_full_name_obj = None
                            cnt_not1 = 0

                            try:
                                item_line = Item_line.objects.get(parent=item_ref.parent, child=item_ref.child)
                                section = item_line.section

                                if section and section != 'Документация':
                                    if IsBitOn(props, 6) and not item_line.qty:
                                        cnt_not1 += 1
                                        notes.append(f'Не указано количество.')
                                        if not item_full_name:
                                            item_full_name_obj = items_refs_stack.get_full_path_obj
                                            item_full_name = items_refs_stack.get_full_path

                            except Item_line.DoesNotExist:
                                cnt_not += 1
                                notes.append(f'Не входит в детализацию.')
                                if not item_full_name:
                                    item_full_name_obj = items_refs_stack.get_full_path_obj
                                    item_full_name = items_refs_stack.get_full_path

                            if section and section != 'Документация':
                                if operations_cnt == 0:
                                    if IsBitOn(props, 5):
                                        cnt_not += 1
                                        notes.append(f'Не указаны операции.')
                                        item_full_name_obj = items_refs_stack.get_full_path_obj
                                        item_full_name = items_refs_stack.get_full_path
                                else:
                                    for operation in Operations_item.objects.filter(item=item_ref.child):
                                        if IsBitOn(props, 0) and not operation.qty:
                                            cnt_not1 = 1
                                            notes.append(f'Операция: {operation.operation.full_name} не указана длительность.')
                                            if not item_full_name:
                                                item_full_name_obj = items_refs_stack.get_full_path_obj
                                                item_full_name = items_refs_stack.get_full_path

                                        if IsBitOn(props, 1) and not operation.num:
                                            cnt_not1 = 1
                                            notes.append(f'Операция: {operation.operation.full_name} не указан № п/п.')
                                            if not item_full_name:
                                                item_full_name_obj = items_refs_stack.get_full_path_obj
                                                item_full_name = items_refs_stack.get_full_path

                                        if IsBitOn(props, 2):
                                            operation_material_cnt = Operation_material.objects.filter(operationitem=operation).count()
                                            if operation_material_cnt == 0:
                                                cnt_not1 = 1
                                                notes.append(f'Операция: {operation.operation.full_name} не указаны материалы или стандартные изделия.')
                                                if not item_full_name:
                                                    item_full_name_obj = items_refs_stack.get_full_path_obj
                                                    item_full_name = items_refs_stack.get_full_path

                                        if IsBitOn(props, 3):
                                            operation_resources_cnt = Operation_resources.objects.filter(operationitem=operation).count()
                                            if operation_resources_cnt == 0:
                                                cnt_not1 = 1
                                                notes.append(f'Операция: {operation.operation.full_name} не указан ресурс либо место выполнения.')
                                                if not item_full_name:
                                                    item_full_name_obj = items_refs_stack.get_full_path_obj
                                                    item_full_name = items_refs_stack.get_full_path

                                        if IsBitOn(props, 4) and not operation.ed_izm:
                                            cnt_not1 = 1
                                            notes.append(f'Операция: {operation.operation.full_name} не указана единица измерения.')
                                            if not item_full_name:
                                                item_full_name_obj = items_refs_stack.get_full_path_obj
                                                item_full_name = items_refs_stack.get_full_path

                            if len(notes) > 0:
                                notes_str = "\n".join(notes)
                                Ready_2_launch_detail.objects.get_or_create(
                                    ready=ready_2_launch,
                                    notes=notes_str,
                                    item_full_name=item_full_name,
                                    item_full_name_obj=item_full_name_obj,
                                )
                            cnt_not += cnt_not1
                            step_res = progress.step()
                            cnt += 1
                            if step_res != 0:
                                raise ProgressDroped(progress_deleted)

                        ready = round(100 - cnt_not / cnt * 100, 2)
                        all_notes.append(f'{ready}%')
                        notes_str = "\n".join(all_notes)
                        ready_2_launch.notes = f'<pre>{notes_str}</pre>'
                        ready_2_launch.save()

                        options_str = "\n" + "\n".join(options)
                        settings.EVENT_STACK.EVENTS_PRODUCTION_READY_2_LAUNCH.send_message(f'Выполнена {demand_str} <h3>готовность: {ready} </h3>{options_str}<p/>')
            settings.LOCKS.release(f'Ready_2_launchManager.make_{demand.id}')
        except ProgressDroped as ex:
            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
            settings.LOCKS.release(f'Ready_2_launchManager.make_{demand.id}')
            raise ex


    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        demand_id = _data.get('demand_id')
        delAttr(_data, 'demand__date')
        delAttr(_data, 'demand__name')
        delAttr(_data, 'demand__code')
        delAttr(_data, 'demand__description')
        delAttr(_data, 'demand__precent_item__STMP_1__value_str')
        delAttr(_data, 'demand__precent_item__STMP_2__value_str')
        delAttr(_data, 'demand__precent_item__STMP_1_id')
        delAttr(_data, 'demand__precent_item__STMP_2_id')
        delAttr(_data, 'demand__precent_item_id')

        props = 0
        if _data.get('check_qty'):
            props = TurnBitOn(props, 0)
        delAttr(_data, 'check_qty')

        if _data.get('check_num'):
            props = TurnBitOn(props, 1)
        delAttr(_data, 'check_num')

        if _data.get('check_material'):
            props = TurnBitOn(props, 2)
        delAttr(_data, 'check_material')

        if _data.get('check_resources'):
            props = TurnBitOn(props, 3)
        delAttr(_data, 'check_resources')

        if _data.get('check_edizm'):
            props = TurnBitOn(props, 4)
        delAttr(_data, 'check_edizm')

        if _data.get('check_operation'):
            props = TurnBitOn(props, 5)
        delAttr(_data, 'check_operation')

        if _data.get('check_colvo'):
            props = TurnBitOn(props, 6)
        delAttr(_data, 'check_colvo')

        setAttr(_data, 'props', props)

        with transaction.atomic():
            # date = StrToDate(_data.get('date'))
            # setAttr(_data, 'lastmodified', date)
            delAttr(_data, 'date')
            res = super().create(**_data)
            Ready_2_launchManager.make(
                demand=demand_id,
                user=request.user_id,
                ready_2_launch=res,
                props=props
            )

            res = Ready_2_launchManager.getRecord(res)
            data.update(DelProps(res))
        return data

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        return data

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'date': record.lastmodified,
            'demand_id': record.demand.id,
            'demand__date': record.demand.date,
            'demand__code': record.demand.code,
            'demand__name': record.demand.name,
            'check_qty': IsBitOn(record.props, 0),
            'check_num': IsBitOn(record.props, 1),
            'check_material': IsBitOn(record.props, 2),
            'check_resources': IsBitOn(record.props, 3),
            'check_edizm': IsBitOn(record.props, 4),
            'check_operation': IsBitOn(record.props, 5),
            'check_colvo': IsBitOn(record.props, 6),
            'demand__description': record.demand.description,
            'demand__precent_item_id': record.demand.precent_item.id,
            'demand__precent_item__STMP_1_id': record.demand.precent_item.item.STMP_1.id if record.demand.precent_item.item.STMP_1 else None,
            'demand__precent_item__STMP_1__value_str': record.demand.precent_item.item.STMP_1.value_str if record.demand.precent_item.item.STMP_1 else None,
            'demand__precent_item__STMP_2_id': record.demand.precent_item.item.STMP_2.id if record.demand.precent_item.item.STMP_2 else None,
            'demand__precent_item__STMP_2__value_str': record.demand.precent_item.item.STMP_2.value_str if record.demand.precent_item.item.STMP_2 else None,
            'notes': record.notes,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return DelProps(res)

    def get_queryset(self):
        return Ready_2_launchQuerySet(self.model, using=self._db)


class Ready_2_launch(AuditModel):
    demand = ForeignKeyCascade(Demand)
    notes = TextField(null=True, blank=True)
    props = BitField(flags=(
        ('check_qty', 'Проверять длительность'),  # 1
        ('check_num', 'Проверять № п/п'),  # 2
        ('check_material', 'Проверять материалы'),  # 4
        ('check_resources', 'Проверять ресурсы'),  # 8
        ('check_edizm', 'Проверять еденицу измерения'),  # 16
        ('check_operation', 'Проверять операцию'),  # 32
        ('check_colvo', 'Проверять количество'),  # 64
    ), default=0, db_index=True)

    objects = Ready_2_launchManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Готовность к запуску'



# Generated by Django 3.0.3 on 2020-02-27 16:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0217_operations_template_detail_description'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operations_template_detail',
            unique_together={('operation', 'template')},
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import connection, transaction
from tqdm import tqdm

from kaf_pas.production.models.operation_material import Operation_material
from kaf_pas.production.models.operation_resources import Operation_resources
from kaf_pas.production.models.operations_item import Operations_item
from kaf_pas.production.models.resource_users import Resource_users

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Удаление дубликатов товарных позиций"

    def handle(self, *args, **options):
        with transaction.atomic():
            with connection.cursor() as cursor:
                cursor.execute(f'''select count(*), item_id, operation_id
                                    from production_operations_item
                                    group by item_id, operation_id
                                    having count(*) > 1
                                        ''')
                rows = cursor.fetchall()
                pbar = tqdm(total=len(rows))
                for row in rows:
                    count, item_id, operation_id = row

                    first_step = True
                    _operations_item = None

                    for operations_item in Operations_item.objects.filter(item_id=item_id, operation_id=operation_id):
                        if not first_step:
                            for operation_resources in Operation_resources.objects.filter(operationitem=operations_item):
                                operation_resources.operations_item = _operations_item
                                operation_resources.save()

                            for operation_resources in Operation_material.objects.filter(operationitem=operations_item):
                                operation_resources.operations_item = _operations_item
                                operation_resources.save()

                            operations_item.delete()
                        else:
                            _operations_item = operations_item
                            first_step = False
                    pbar.update()
                pbar.close()



from django.urls import path

from kaf_pas.planing.views import operation_item

urlpatterns = [

    path('Operation_item/Fetch/', operation_item.Operation_item_Fetch),
    path('Operation_item/Fetch1/', operation_item.Operation_item_Fetch1),
    path('Operation_item/Add', operation_item.Operation_item_Add),
    path('Operation_item/Update', operation_item.Operation_item_Update),
    path('Operation_item/Remove', operation_item.Operation_item_Remove),
    path('Operation_item/Lookup/', operation_item.Operation_item_Lookup),
    path('Operation_item/Info/', operation_item.Operation_item_Info),
    path('Operation_item/Copy', operation_item.Operation_item_Copy),

]



# Generated by Django 3.0.4 on 2020-03-05 17:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0228_remove_launch_item_line_spc_clm_count_per_one'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='resource',
            name='Resource_unique_constraint_1',
        ),
    ]



import logging

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.uploades import Uploades, UploadesManager

logger = logging.getLogger(__name__)


class Uploades_documentsQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Uploades_documentsManager(AuditManager):

    def deleteFromRequest(self, request, removed=None, ):
        from isc_common.auth.models.user import User

        ids = request.GET.getlist('ids')
        user = User.objects.get(username=request.GET.get('ws_channel').split('_')[1])

        res = 0
        doc_cnt = 0
        lotsman_cnt = 0

        for i in range(0, len(ids), 2):
            id = ids[i]
            visibleMode = ids[i + 1]

            if visibleMode != "none":
                res += super().filter(id=id).soft_delete(visibleMode=visibleMode)
            else:
                res, _doc_cnt, _lotsman_cnt = UploadesManager.del_upload(
                    id=Uploades_documents.objects.get(id=id).upload.id,
                    document_id=id,
                    user=user,
                    doc_cnt=doc_cnt,
                    last_step=len(ids) / 2 == i,
                    lotsman_cnt=lotsman_cnt
                )
                doc_cnt += _doc_cnt
                lotsman_cnt += _lotsman_cnt
        return res

    @staticmethod
    def getRecord(record):
        document_view = record.document_view
        res = {
            'id': record.id,
            'document_id': record.document.id,
            'document__path_id': record.document.path.id,
            'document__attr_type_id': record.document.attr_type.id,
            'document__attr_type__code': record.document.attr_type.code,
            'document__attr_type__name': record.document.attr_type.name,
            'document__file_document': record.document.file_document,
            'document__file_size': record.document.file_size,
            'document__lastmodified': record.document.lastmodified,
            'document__file_modification_time': record.document.file_modification_time,
            'document__file_access_time': record.document.file_access_time,
            'document__file_change_time': record.document.file_change_time,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Uploades_documentsQuerySet(self.model, using=self._db)


class Uploades_documents(AuditModel):
    upload = ForeignKeyProtect(Uploades)
    document = ForeignKeyProtect(Documents)

    objects = Uploades_documentsManager()

    def __str__(self):
        return f'{self.id}'

    class Meta:
        unique_together = (('upload', 'document'),)



import logging

from bitfield import BitField
from django.conf import settings
from django.db import transaction, connection
from django.db.models import PositiveIntegerField, OneToOneField, PROTECT, CASCADE, UniqueConstraint, Q
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.bit import IsBitOn
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade, ForeignKeySetNull
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from isc_common.number import DelProps
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.production.models.launch_operations_item import Launch_operations_item
from kaf_pas.production.models.operation_resources import Operation_resources
from kaf_pas.production.models.resource import Resource

logger = logging.getLogger(__name__)


class Launch_operation_resourcesQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        if kwargs.get('resource'):
            setAttr(kwargs, 'resource_id', kwargs.get('resource').id)
            delAttr(kwargs, 'resource')

        if kwargs.get('location'):
            setAttr(kwargs, 'location_id', kwargs.get('location').id)
            delAttr(kwargs, 'location')

        if not kwargs.get('resource_id') and not kwargs.get('location_id'):
            raise Exception('Необходим хотябы один выбранный параметр.')
        delAttr(kwargs, 'resource__name')
        delAttr(kwargs, 'location__full_name')

        resource_id = kwargs.get('resource_id')
        if resource_id:
            setAttr(kwargs, 'location_id', Resource.objects.get(id=resource_id).location.id)

        setAttr(kwargs, '_operation', 'create')
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Launch_operation_resourcesManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def refresh_resource(apps=None, schema_editor=None):
        settings.LOCKS.acquire('Launch_operation_resourcesManager.refresh_resource')
        sql_text = '''select  launch_operationitem_id, count(*)
                from production_launch_operation_resources
                group by launch_operationitem_id
                having count(*) > 1'''

        sql_items = f'''select id
        from production_launch_operation_resources
        where launch_operationitem_id = %s'''

        with transaction.atomic():
            with connection.cursor() as cursor:
                cursor.execute(sql_text)
                rows = cursor.fetchall()
                for row in rows:
                    launch_operationitem_id, count = row
                    # print(f'launch_operationitem_id: {launch_operationitem_id}')
                    cursor.execute(sql_items, [launch_operationitem_id])
                    rows_item = cursor.fetchall()

                    first_step = True
                    for item in rows_item:
                        id, = item
                        if not first_step:
                            # print(f'id: {id}')
                            res = Launch_operation_resources.objects.filter(id=id).delete()
                            # print(res)
                        else:
                            first_step = False
        settings.LOCKS.release('Launch_operation_resourcesManager.refresh_resource')

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        operationitem_id = _data.get('operationitem_id')

        res = []
        if isinstance(operationitem_id, list):
            delAttr(_data, 'operationitem_id')
            with transaction.atomic():
                for operationitem in operationitem_id:
                    setAttr(_data, 'operationitem_id', operationitem)
                    operation_resource = super().create(**_data)
                    _res = model_to_dict(operation_resource)
                    setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
                    setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
                    res.append(DelProps(_res))
        else:
            operation_resource = super().create(**_data)
            _res = model_to_dict(operation_resource)
            setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
            setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
            res.append(DelProps(_res))
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'operation_resources_id': record.operation_resources.id if record.operation_resources else None,
            'launch_operationitem_id': record.launch_operationitem.id,
            'resource_id': record.resource.id if record.resource else None,
            'resource__code': record.resource.code if record.resource else None,
            'resource__name': record.resource.name if record.resource else None,
            'location_id': record.location.id if record.location else None,
            'location__code': record.location.code if record.location else None,
            'location__name': record.location.name if record.location else None,
            'location__full_name': record.location.full_name if record.location else None,
            'complex_name': record.complex_name,
            'batch_size': record.batch_size,
            'props': record.props._value,
            'enabled': IsBitOn(record.props, 0),
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Launch_operation_resourcesQuerySet(self.model, using=self._db)


class Launch_operation_resources(AuditModel):
    operation_resources = ForeignKeySetNull(Operation_resources, null=True, blank=True)
    launch_operationitem = OneToOneField(Launch_operations_item, on_delete=CASCADE)
    resource = ForeignKeyProtect(Resource, null=True, blank=True)
    location = ForeignKeyProtect(Locations, null=True, blank=True)
    batch_size = PositiveIntegerField(default=1, db_index=True)

    props = BitField(flags=(
        ('enabled', 'Доступность в данной производственной спецификации'),  # 1
    ), default=1, db_index=True)

    @property
    def complex_name(self):
        return f'{self.resource.location.full_name}{self.resource.name}' if self.resource else None

    objects = Launch_operation_resourcesManager()

    def __str__(self):
        return f"{self.id}, operationitem: [{self.launch_operationitem}], resource: [{self.resource}], location: [{self.location}]"

    class Meta:
        verbose_name = 'Кросс таблица'
        constraints = [
            UniqueConstraint(fields=['launch_operationitem'], condition=Q(location=None) & Q(resource=None), name='Launch_operation_resources_unique_constraint_0'),
            UniqueConstraint(fields=['launch_operationitem', 'resource'], condition=Q(location=None), name='Launch_operation_resources_unique_constraint_1'),
            UniqueConstraint(fields=['launch_operationitem', 'location'], condition=Q(resource=None), name='Launch_operation_resources_unique_constraint_2'),
            UniqueConstraint(fields=['launch_operationitem', 'location', 'resource'], name='Launch_operation_resources_unique_constraint_3'),
        ]



# Generated by Django 3.0.3 on 2020-02-13 12:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0178_auto_20200210_0856'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='launches',
            name='props',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-17 09:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0092_auto_20200217_0914'),
    ]

    operations = [
        migrations.AlterField(
            model_name='levels',
            name='code',
            field=models.PositiveIntegerField(),
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operations

urlpatterns = [

    path('Operations_plan/Fetch/', operations.Operations_Fetch),
    path('Operations_plan/Fetch_Facets/', operations.Operations_Fetch_Facets),
    path('Operations_plan/Add', operations.Operations_Add),
    path('Operations_plan/Update', operations.Operations_Update),
    path('Operations_plan/Remove', operations.Operations_Remove),
    path('Operations_plan/Lookup/', operations.Operations_Lookup),
    path('Operations_plan/Info/', operations.Operations_Info),
    path('Operations_plan/Copy', operations.Operations_Copy),

]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.input.models.user_add_info import User_add_infoManager, User_add_info


@JsonResponseWithException()
def User_add_info_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=User_add_info.objects.
                filter().
                get_range_rows1(
                request=request,
                function=User_add_infoManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_add_info_Add(request):
    return JsonResponse(DSResponseAdd(data=User_add_info.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_add_info_Update(request):
    return JsonResponse(DSResponseUpdate(data=User_add_info.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_add_info_Remove(request):
    return JsonResponse(DSResponse(request=request, data=User_add_info.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_add_info_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=User_add_info.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_add_info_Info(request):
    return JsonResponse(DSResponse(request=request, data=User_add_info.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



from isc_common.http.DSResponse import JsonResponseWithException, DSResponse, DSResponseAdd, DSResponseUpdate
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse

from kaf_pas.ckk.models.attr_type import Attr_type, AttrManager


@JsonResponseWithException()
def Attr_type_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Attr_type.objects.
                filter().
                get_range_rows1(
                request=request,
                function=AttrManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Attr_type_Add(request):
    return JsonResponse(DSResponseAdd(data=Attr_type.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Attr_type_Update(request):
    return JsonResponse(DSResponseUpdate(data=Attr_type.objects.updateFromRequest(request, removed=['isFolder']), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Attr_type_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Attr_type.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Attr_type_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Attr_type.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Attr_type_Info(request):
    return JsonResponse(DSResponse(request=request, data=Attr_type.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2 on 2019-04-25 19:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0008_auto_20190425_1925'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_plan_template_locations',
            name='item_line',
        ),
    ]



import logging

from django.conf import settings
from django.db import transaction
from django.forms import model_to_dict

from isc_common import delAttr
from isc_common.http.DSRequest import DSRequest
from isc_common.number import DelProps
from kaf_pas.planing.models.operations import Operations, OperationsManager, OperationsQuerySet
from kaf_pas.planing.models.operations_view import Operations_view

logger = logging.getLogger(__name__)


class Posting_itemsQuerySet(OperationsQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Posting_itemsManager(OperationsManager):
    def createFromRequest(self, request, removed=None):
        from kaf_pas.accounting.models.tmp_buffer import Tmp_buffer
        from kaf_pas.planing.models.operation_color import Operation_color
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_value import Operation_value

        request = DSRequest(request=request)
        data = request.get_data()
        user_id = request.user_id
        items = data.get('items')
        if isinstance(items, list):
            delAttr(data, 'items')
            _data = data.copy()
            _data.setdefault('opertype', settings.OPERS_TYPES_STACK.POSTING_TASK)
            _data.setdefault('creator_id', user_id)

            with transaction.atomic():
                parent = Operations.objects.create(**_data)
                Operation_refs.objects.create(child=parent)

                for item in items:
                    child = dict()
                    child.setdefault('date', parent.date)
                    child.setdefault('creator_id', user_id)
                    child.setdefault('opertype', settings.OPERS_TYPES_STACK.POSTING_DETAIL_TASK)
                    child.setdefault('status', settings.OPERS_TYPES_STACK.POSTING_DETAIL_TASK_STATUSES.get('new'))
                    child = Operations.objects.create(**child)

                    Operation_refs.objects.create(parent=parent, child=child)

                    if item.get('item_id'):
                        Operation_item.objects.create(operation=child, item_id=item.get('item_id'))

                    if item.get('value'):
                        Operation_value.objects.create(operation=child, value=item.get('value'), edizm_id=item.get('edizm_id'))

                    if item.get('color_id'):
                        Operation_color.objects.create(operation=child, color_id=item.get('color_id'))

                Tmp_buffer.objects.filter(user_id=user_id).delete()

                parent = model_to_dict(parent)
                return DelProps(parent)
        else:
            return {}

    def get_queryset(self):
        return Posting_itemsQuerySet(self.model, using=self._db)


class Posting_items(Operations_view):
    objects = Posting_itemsManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        proxy = True
        verbose_name = 'Оприходования'



import logging

from django.db.models import PositiveIntegerField

from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet

logger = logging.getLogger(__name__)


class LevelsQuerySet(BaseRefQuerySet):
    pass


class LevelsManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return LevelsQuerySet(self.model, using=self._db)


class Levels(BaseRef):
    code = PositiveIntegerField()
    objects = LevelsManager()

    def __str__(self):
        return f'ID:{self.id}, code: {self.code}, name: {self.name}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Уровни производственного цикла'



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_ZONE

urlpatterns = [

    path('Document_attributes_SPC_CLM_ZONE/Fetch/', document_attributes_SPC_CLM_ZONE.Document_attributes_SPC_CLM_ZONE_Fetch),
    path('Document_attributes_SPC_CLM_ZONE/Add', document_attributes_SPC_CLM_ZONE.Document_attributes_SPC_CLM_ZONE_Add),
    path('Document_attributes_SPC_CLM_ZONE/Update', document_attributes_SPC_CLM_ZONE.Document_attributes_SPC_CLM_ZONE_Update),
    path('Document_attributes_SPC_CLM_ZONE/Remove', document_attributes_SPC_CLM_ZONE.Document_attributes_SPC_CLM_ZONE_Remove),
    path('Document_attributes_SPC_CLM_ZONE/Lookup/', document_attributes_SPC_CLM_ZONE.Document_attributes_SPC_CLM_ZONE_Lookup),
    path('Document_attributes_SPC_CLM_ZONE/Info/', document_attributes_SPC_CLM_ZONE.Document_attributes_SPC_CLM_ZONE_Info),
    path('Document_attributes_SPC_CLM_ZONE/Copy', document_attributes_SPC_CLM_ZONE.Document_attributes_SPC_CLM_ZONE_Copy),

]



# Generated by Django 2.2.6 on 2019-10-22 07:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
        ('production', '0075_auto_20191022_0721'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together={('prod_item', 'item', 'num')},
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from isc_common.logger.Logger import Logger
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_ext import DocumentManagerExt
from kaf_pas.kd.models.pdfs import Pdfs

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Command(BaseCommand):
    help = "Test Make PDF"
    documentManagerExt = DocumentManagerExt(logger=logger)

    def handle(self, *args, **options):
        pdfs = [pdfs for pdfs in Pdfs.objects.filter(props=~Documents.props.beenItemed) if pdfs.file_document.lower().find('мусор') == -1]

        with transaction.atomic():
            STMP_1_type = Attr_type.objects.get(code='STMP_1')
            STMP_2_type = Attr_type.objects.get(code='STMP_2')

            pbar = tqdm(total=len(pdfs))
            for document in pdfs:
                self.documentManagerExt.make_pdf(document=document, STMP_1_type=STMP_1_type, STMP_2_type=STMP_2_type)
                pbar.update()
            pbar.close()



# Generated by Django 2.2 on 2019-04-14 17:01

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0018_auto_20190414_1616'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item_line',
            name='item',
        ),
        migrations.AddField(
            model_name='item_line',
            name='item_child',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='item_parent',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, related_name='item_parent', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.DeleteModel(
            name='Item_refs',
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-18 14:45

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0057_dogovors'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dogovors',
            name='document',
        ),
        migrations.AlterField(
            model_name='precent',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='document_p', to='sales.Dogovors'),
        ),
    ]



# import requests
# from django.http import HttpResponseRedirect
#
# from isc_common.http.DSRequest import DSRequest
# from django.conf import settings
#
#
# def uploadKompasFile(request):
#     _request = DSRequest(request=request)
#     data = _request.get_data()
#
#     id = data.get('id')
#     userId = data.get('userId')
#     wsHost = data.get('wsHost')
#     wsPort = data.get('wsPort')
#     wsChannel = data.get('wsChannel')
#
#     r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Documents/UploadKompas', params=dict(id=id, wsPort=wsPort, wsHost=wsHost, wsChannel=wsChannel, userId=userId))
#     return HttpResponseRedirect(r.url)
#
#
# def uploadPdfFile(request):
#     return HttpResponseRedirect(f'{settings.KOMPAS_INFORMICA}/logic/Documents/UploadPdf/')



# Generated by Django 3.0.3 on 2020-02-11 08:50

from django.db import migrations, models
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0208_auto_20200210_1259'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='item_refs',
            constraint=models.CheckConstraint(check=models.Q(_negated=True, child=django.db.models.expressions.F('parent')), name='Item_refs_not_circulate_refs'),
        ),
    ]



import logging

from bitfield import BitField

from isc_common.models.base_ref import BaseRefManager, BaseRefQuerySet, BaseRefHierarcy
from isc_common.number import DelProps

logger = logging.getLogger(__name__)


class Operation_typesQuerySet(BaseRefQuerySet):
    pass


class Operation_typesManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'full_name': record.full_name,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
            'props': record.props,
            'plus': record.props.plus,
            'minus': record.props.minus,
        }
        return DelProps(res)

    def get_queryset(self):
        return Operation_typesQuerySet(self.model, using=self._db)


class Operation_types(BaseRefHierarcy):
    props = BitField(flags=(
        ('plus', 'Приходная операция'),  # 2
        ('minus', 'Расходная операция'),  # 2
    ), default=0, db_index=True)

    objects = Operation_typesManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, full_name: {self.full_name}, parent_id: {self.parent_id}, props: {self.props}, description: {self.description}"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Типы системных операций'




# Generated by Django 3.0.3 on 2020-02-22 10:30

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0106_auto_20200222_1028'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='planing.Status_operation_types'),
        ),
        migrations.AlterModelTable(
            name='operations_view',
            table='planing_operations_view',
        ),
    ]



from django.urls import path

from kaf_pas.accounting.views import tmp_buffer_off

urlpatterns = [

    path('TmpBuffers_off/Fetch/', tmp_buffer_off.Tmp_buffer_off_Fetch),
    path('TmpBuffers_off/Add', tmp_buffer_off.Tmp_buffer_off_Add),
    path('TmpBuffers_off/Update', tmp_buffer_off.Tmp_buffer_off_Update),
    path('TmpBuffers_off/Remove', tmp_buffer_off.Tmp_buffer_off_Remove),
    path('TmpBuffers_off/Lookup/', tmp_buffer_off.Tmp_buffer_off_Lookup),
    path('TmpBuffers_off/Info/', tmp_buffer_off.Tmp_buffer_off_Info),
    path('TmpBuffers_off/Copy', tmp_buffer_off.Tmp_buffer_off_Copy),

]



from django.urls import path

from kaf_pas.ckk.views import item_qty_view

urlpatterns = [

    path('Item_qty_view/Fetch/', item_qty_view.Item_qty_view_Fetch),
    path('Item_qty_view/Add', item_qty_view.Item_qty_view_Add),
    path('Item_qty_view/Update', item_qty_view.Item_qty_view_Update),
    path('Item_qty_view/Remove', item_qty_view.Item_qty_view_Remove),
    path('Item_qty_view/Lookup/', item_qty_view.Item_qty_view_Lookup),
    path('Item_qty_view/Info/', item_qty_view.Item_qty_view_Info),
    path('Item_qty_view/Copy', item_qty_view.Item_qty_view_Copy),

]



# Generated by Django 2.2.2 on 2019-06-07 18:52

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0034_document_attr_cross_document_attributes'),
        ('ckk', '0079_auto_20190607_1721'),
    ]

    operations = [
        migrations.AddField(
            model_name='item_line',
            name='STMP_1',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='STMP_1', to='kd.Document_attributes', verbose_name='Наименование'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='STMP_2',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='STMP_1', to='kd.Document_attributes', verbose_name='Обозначение'),
        ),
        migrations.AlterUniqueTogether(
            name='item_line',
            unique_together={('parent', 'child'), ('STMP_1', 'STMP_2')},
        ),
        migrations.RemoveField(
            model_name='item_line',
            name='SPC_CLM_MARK',
        ),
        migrations.RemoveField(
            model_name='item_line',
            name='SPC_CLM_NAME',
        ),
    ]



# Generated by Django 2.2.1 on 2019-06-05 10:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0047_auto_20190530_1353'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='resource',
            name='category',
        ),
        migrations.DeleteModel(
            name='Resource_categories',
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-22 12:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0054_auto_20190521_2304'),
        ('production', '0021_auto_20190522_1242'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operation_plan_templates',
            unique_together={('code', 'item')},
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-21 10:16

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0073_auto_20200121_0739'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_refs',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='operation_child', to='planing.Operations'),
        ),
        migrations.AlterField(
            model_name='operation_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='operation_parent', to='planing.Operations'),
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import item_refs_hierarcy

urlpatterns = [

    path('Item_refs_hierarcy/Fetch/', item_refs_hierarcy.Item_refs_hierarcy_Fetch),
    path('Item_refs_hierarcy/Add', item_refs_hierarcy.Item_refs_hierarcy_Add),
    path('Item_refs_hierarcy/Update', item_refs_hierarcy.Item_refs_hierarcy_Update),
    path('Item_refs_hierarcy/Remove', item_refs_hierarcy.Item_refs_hierarcy_Remove),
    path('Item_refs_hierarcy/Lookup/', item_refs_hierarcy.Item_refs_hierarcy_Lookup),
    path('Item_refs_hierarcy/Info/', item_refs_hierarcy.Item_refs_hierarcy_Info),
    path('Item_refs_hierarcy/Copy', item_refs_hierarcy.Item_refs_hierarcy_Copy),

]



# Generated by Django 2.2.6 on 2019-10-28 20:25

from django.db import migrations

import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0097_auto_20191028_1750'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dogovors',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operation_types_users

urlpatterns = [

    path('Operation_types_users/Fetch/', operation_types_users.Operation_types_users_Fetch),
    path('Operation_types_users/Add', operation_types_users.Operation_types_users_Add),
    path('Operation_types_users/Update', operation_types_users.Operation_types_users_Update),
    path('Operation_types_users/Remove', operation_types_users.Operation_types_users_Remove),
    path('Operation_types_users/Lookup/', operation_types_users.Operation_types_users_Lookup),
    path('Operation_types_users/Info/', operation_types_users.Operation_types_users_Info),
    path('Operation_types_users/Copy', operation_types_users.Operation_types_users_Copy),

]



# Generated by Django 3.0.2 on 2020-01-17 16:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0057_auto_20191209_1131'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='operation_launches',
            options={'verbose_name': 'Кросс таблица'},
        ),
    ]



# Generated by Django 2.2 on 2019-04-14 08:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0013_auto_20190414_0801'),
    ]

    operations = [
        migrations.AddField(
            model_name='attr_type',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='item',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='item_line_refs',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.ed_izm import Ed_izm, Ed_izmManager


@JsonResponseWithException()
def Ed_izm_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Ed_izm.objects.
                get_range_rows1(
                request=request,
                function=Ed_izmManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ed_izm_Add(request):
    return JsonResponse(DSResponseAdd(data=Ed_izm.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ed_izm_Update(request):
    return JsonResponse(DSResponseUpdate(data=Ed_izm.objects.updateFromRequest(request, removed=['isFolder']), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ed_izm_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Ed_izm.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ed_izm_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Ed_izm.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ed_izm_Info(request):
    return JsonResponse(DSResponse(request=request, data=Ed_izm.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.8 on 2019-12-04 20:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0144_auto_20191204_2013'),
    ]

    operations = [
        migrations.AddField(
            model_name='launch_operations_item',
            name='q_paralel',
            field=models.PositiveIntegerField(default=None),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.production.models.operations import Operations
from kaf_pas.production.models.operations_view import Operations_view, Operations_viewManager


@JsonResponseWithException()
def Operations_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Operations_view.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Operations_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_Add(request):
    return JsonResponse(DSResponseAdd(data=Operations.objects.createFromRequest(request=request, propsArr=['made_common_form', 'assemly']), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_Update(request):
    return JsonResponse(DSResponseUpdate(data=Operations.objects.updateFromRequest(request, propsArr=['assemly', 'made_common_form']), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Operations.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Operations.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_Info(request):
    return JsonResponse(DSResponse(request=request, data=Operations.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_KOD

urlpatterns = [

    path('Document_attributes_SPC_CLM_KOD/Fetch/', document_attributes_SPC_CLM_KOD.Document_attributes_SPC_CLM_KOD_Fetch),
    path('Document_attributes_SPC_CLM_KOD/Add', document_attributes_SPC_CLM_KOD.Document_attributes_SPC_CLM_KOD_Add),
    path('Document_attributes_SPC_CLM_KOD/Update', document_attributes_SPC_CLM_KOD.Document_attributes_SPC_CLM_KOD_Update),
    path('Document_attributes_SPC_CLM_KOD/Remove', document_attributes_SPC_CLM_KOD.Document_attributes_SPC_CLM_KOD_Remove),
    path('Document_attributes_SPC_CLM_KOD/Lookup/', document_attributes_SPC_CLM_KOD.Document_attributes_SPC_CLM_KOD_Lookup),
    path('Document_attributes_SPC_CLM_KOD/Info/', document_attributes_SPC_CLM_KOD.Document_attributes_SPC_CLM_KOD_Info),
    path('Document_attributes_SPC_CLM_KOD/Copy', document_attributes_SPC_CLM_KOD.Document_attributes_SPC_CLM_KOD_Copy),

]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.item_line import Item_line
from kaf_pas.ckk.models.item_line_view import Item_line_view, Item_line_viewManager


@JsonResponseWithException()
def Item_line_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Item_line_view.objects.
                filter().
                # select_related(
                # 'SPC_CLM_FORMAT',
                # 'SPC_CLM_ZONE',
                # 'SPC_CLM_POS',
                # 'SPC_CLM_MARK',
                # 'SPC_CLM_NAME',
                # 'SPC_CLM_COUNT',
                # 'SPC_CLM_NOTE',
                # 'SPC_CLM_MASSA',
                # 'SPC_CLM_MATERIAL',
                # 'SPC_CLM_USER',
                # 'SPC_CLM_KOD',
                # 'SPC_CLM_FACTORY', ).
                get_range_rows1(
                request=request,
                function=Item_line_viewManager.getRecord,
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_line_Add(request):
    return JsonResponse(DSResponseAdd(data=Item_line.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_line_Update(request):
    return JsonResponse(DSResponseUpdate(data=Item_line.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Item_line_Copy(request):
    return JsonResponse(DSResponseUpdate(data=Item_line.objects.copyFromRequest(request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Item_line_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Item_line.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_line_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Item_line.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_line_Info(request):
    return JsonResponse(DSResponse(request=request, data=Item_line.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)

# @JsonResponseWithException()
# def Item_line_Copy(request):
#     return JsonResponse(DSResponse(request=request, data=Item_line.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.6 on 2019-10-31 17:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0084_auto_20191014_1326'),
    ]

    operations = [
        migrations.RunSQL(
            sql=r'CREATE INDEX "kd_document_attributes_value_str_idx" ON "kd_document_attributes" (UPPER("value_str"));',
            reverse_sql=r'DROP INDEX "kd_document_attributes_value_str_idx";'
        )
    ]



# Generated by Django 2.2.7 on 2019-11-17 17:58

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0095_launch_item_refs'),
    ]

    operations = [
        migrations.AddField(
            model_name='launch_item_line',
            name='launch',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Launches'),
        ),
        migrations.AddField(
            model_name='launch_item_refs',
            name='launch',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Launches'),
        ),
        migrations.AlterField(
            model_name='launch_item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='Launch_item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='launch_item_line',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='Launch_item_parent', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='launch_item_refs',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='launch_child', to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='launch_item_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='launch_parent', to='ckk.Item'),
        ),
        migrations.DeleteModel(
            name='Launch_items',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-20 10:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0143_files_askon'),
    ]

    operations = [
        migrations.AlterField(
            model_name='material_askon',
            name='field0',
            field=models.CharField(db_index=True, max_length=255, verbose_name='Информация в дерево материалов'),
        ),
    ]



import logging

from django.db import ProgrammingError

logger = logging.getLogger(__name__)


class Operation_typesStack:

    def get_first_item_of_tuple(self, tp):
        res, _ = tp
        return res

    def __init__(self):
        from kaf_pas.planing.models.operation_types import Operation_types
        from kaf_pas.planing.models.status_operation_types import Status_operation_typesManager
        from kaf_pas.planing.models.status_operation_types import Status_operation_types

        try:
            self.PRODUCTION_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='PRD_TSK', defaults=dict(
                props=0,
                name='Задание на производство',
                editing=False,
                deliting=False,
            )))

            self.PRODUCTION_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.PRODUCTION_TASK,
                status_map=[
                    dict(code='new', name='Новый', color='blue'),
                    dict(code='ready_2_start', name='Готов к запуску', color='green', props=Status_operation_types.props.blink),
                    dict(code='not_ready_2_start', name='К запуску не готов', color='red'),
                    dict(code='in_job', name='Просматривается', color='orange'),
                    dict(code='started', name='Запущен', color='blue'),
                    dict(code='started_partly', name='Запущен частично', color='blue', props=Status_operation_types.props.blink),
                    dict(code='transferred', name='Назначенный'),
                    dict(code='doing', name='Выполнен'),
                    dict(code='doing', name='Выполнен'),
                    dict(code='closed', name='Закрыт'),
                ]
            )

            self.PRODUCTION_DETAIL_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='DETAIL_PRD_TSK', defaults=dict(
                props=0,
                name='Детализация Задания на производство',
                editing=False,
                deliting=False,
                parent=self.PRODUCTION_TASK
            )))

            self.PRODUCTION_DETAIL_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.PRODUCTION_DETAIL_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.LAUNCH_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='LAUNCH_TSK', defaults=dict(
                props=0,
                name='Запуск задания на производство',
                editing=False,
                deliting=False,
                parent=self.PRODUCTION_TASK
            )))

            self.LAUNCH_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.LAUNCH_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.MADE_OPERATIONS_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='MADE_OPRS_TSK', defaults=dict(
                props=0,
                name='Выполнение производственной операции',
                editing=False,
                deliting=False,
                parent=self.PRODUCTION_TASK
            )))

            self.MADE_OPERATIONS_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.MADE_OPERATIONS_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.RELEASE_TASK_PLUS = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='RELEASE_TSK_PLS', defaults=dict(
                props=Operation_types.props.plus,
                name='Выпуск изделий',
                editing=False,
                deliting=False,
                parent=self.PRODUCTION_TASK
            )))

            self.RELEASE_TASK_PLUS_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.RELEASE_TASK_PLUS,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.RELEASE_TASK_MINUS = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='RELEASE_TSK_MNS', defaults=dict(
                props=Operation_types.props.minus,
                name='Выпуск изделий',
                editing=False,
                deliting=False,
                parent=self.PRODUCTION_TASK
            )))

            self.RELEASE_TASK_MINUS_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.RELEASE_TASK_MINUS,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.CALC_TASKS = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='CLC_TSK', defaults=dict(
                props=0,
                name='Учет',
                editing=False,
                deliting=False,
            )))

            self.CALC_TASKS_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.CALC_TASKS,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.POSTING_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='PST_TSK', defaults=dict(
                props=0,
                name='Оприходование',
                editing=False,
                deliting=False,
                parent=self.CALC_TASKS
            )))

            self.POSTING_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.POSTING_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.POSTING_DETAIL_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='DETAIL_PST_TSK', defaults=dict(
                props=Operation_types.props.plus,
                name='Детализация Оприходывания',
                editing=False,
                deliting=False,
                parent=self.CALC_TASKS
            )))

            self.POSTING_DETAIL_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.POSTING_DETAIL_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.WRITE_OFF_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='WRT_OFF_TSK', defaults=dict(
                props=0,
                name='Списание',
                editing=False,
                deliting=False,
                parent=self.CALC_TASKS
            )))

            self.WRITE_OFF_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.WRITE_OFF_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.WRITE_DETAIL_OFF_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='DETAIL_WRT_OFF_TSK', defaults=dict(
                props=Operation_types.props.minus,
                name='Детализация Списания',
                editing=False,
                deliting=False,
                parent=self.CALC_TASKS
            )))

            self.WRITE_DETAIL_OFF_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.WRITE_DETAIL_OFF_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )

            self.ROUTING_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='RT_TSK', defaults=dict(
                props=0,
                name='Маршрутизация',
                editing=False,
                deliting=False,
            )))

            self.ROUTING_TASK_STATUSES = Status_operation_typesManager.make_statuses(
                opertype=self.ROUTING_TASK,
                status_map=[
                    dict(code='new', name='Новый'),
                ]
            )


        except ProgrammingError as ex:
            logger.warning(ex)



# Generated by Django 3.0.3 on 2020-03-01 03:54

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0119_delete_operation_attr'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='num',
            field=isc_common.fields.code_field.CodeStrictField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0009_auto_20190820_1236'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user_add_info',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='user_positions',
            name='id_old',
        ),
    ]



from django.db import transaction

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.production.models.operation_material import Operation_material
from kaf_pas.production.models.operation_resources import Operation_resources
from kaf_pas.production.models.operations_item import Operations_item
from kaf_pas.production.models.operations_template_detail import Operations_template_detail, Operations_template_detailManager
from kaf_pas.production.models.operations_template_material import Operations_template_material
from kaf_pas.production.models.operations_template_resource import Operations_template_resource


@JsonResponseWithException()
def Operations_template_detail_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Operations_template_detail.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Operations_template_detailManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_template_detail_Add(request):
    return JsonResponse(DSResponseAdd(data=Operations_template_detail.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_template_detail_Update(request):
    return JsonResponse(DSResponseUpdate(data=Operations_template_detail.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_template_detail_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Operations_template_detail.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_template_detail_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Operations_template_detail.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_template_detail_Info(request):
    return JsonResponse(DSResponse(request=request, data=Operations_template_detail.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_template_detail_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Operations_template_detail.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_template_detail_CopyOpers(request):
    _request = DSRequest(request=request)
    source = _request.json.get('source')
    destination = _request.json.get('destination')

    if source and destination:
        srecords = source.get('records')
        source_name = source.get('source_name')
        drecord = destination.get('record')

        if isinstance(srecords, list) and isinstance(drecord, dict):
            with transaction.atomic():
                for srecord in srecords:
                    if source_name == 'templates':
                        old_operations_item = Operations_template_detail.objects.get(id=srecord.get('id'))
                    else:
                        old_operations_item = Operations_item.objects.get(id=srecord.get('id'))

                    operations_template, created = Operations_template_detail.objects.get_or_create(
                        operation=old_operations_item.operation,
                        template_id=drecord.get('id'),
                        defaults=dict(
                            ed_izm=old_operations_item.ed_izm,
                            qty=old_operations_item.qty,
                            num=old_operations_item.num,
                            description=old_operations_item.description,
                        ))

                    if created:
                        if source_name == 'templates':
                            for operation_resources in Operations_template_resource.objects.filter(template=old_operations_item):
                                Operations_template_resource.objects.get_or_create(
                                    template=operations_template,
                                    resource=operation_resources.resource,
                                    location=operation_resources.location
                                )
                        else:
                            for operation_resources in Operation_resources.objects.filter(operationitem=old_operations_item):
                                Operations_template_resource.objects.get_or_create(
                                    template=operations_template,
                                    resource=operation_resources.resource,
                                    location=operation_resources.location
                                )

                        if source_name == 'templates':
                            for operation_material in Operations_template_material.objects.filter(template=old_operations_item):
                                Operations_template_material.objects.get_or_create(
                                    template=operations_template,
                                    material=operation_material.material,
                                    material_askon=operation_material.material_askon,
                                    edizm=operation_material.edizm,
                                    qty=operation_material.qty,
                                )
                        else:
                            for operation_material in Operation_material.objects.filter(operationitem=old_operations_item):
                                Operations_template_material.objects.get_or_create(
                                    template=operations_template,
                                    material=operation_material.material,
                                    material_askon=operation_material.material_askon,
                                    ed_izm=operation_material.ed_izm,
                                    qty=operation_material.qty,
                                )

        return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)
    else:
        raise Exception(f'Копирование не выполнено.')



import logging

from bitfield import BitField
from django.db import transaction
from django.db.models import UniqueConstraint, Q

from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager, AuditQuerySet
from isc_common.models.base_ref import Hierarcy
from isc_common.number import DelProps
from kaf_pas.ckk.models.locations import Locations

logger = logging.getLogger(__name__)


class Locations_usersQuerySet(AuditQuerySet):
    pass


class Locations_usersManager(AuditManager):

    def createFromRequest(self, request, removed=None):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)
        _data = []

        with transaction.atomic():
            if user_ids and context_ids:
                if not isinstance(user_ids, list):
                    user_ids = [user_ids]

                if not isinstance(context_ids, list):
                    context_ids = [context_ids]

                for user_id in user_ids:
                    for context_id in context_ids:
                        res = super().get_or_create(user_id=user_id, location_id=context_id)

            return data

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        parent_id = data.get('parent_id')

        with transaction.atomic():
            props = data.get('props', 0)
            if data.get('compulsory_reading') == True:
                props |= Locations_users.props.compulsory_reading
            else:
                props &= ~Locations_users.props.compulsory_reading

            _data = dict()
            setAttr(_data, 'props', props)
            setAttr(_data, 'parent_id', parent_id)

            super().update_or_create(id=data.get('id'), defaults=_data)

            return data

    def copyUsersFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        source = data.get('source')
        destination = data.get('destination')

        res = 0
        with transaction.atomic():
            for s in source:
                Locations_users.objects.get_or_create(user_id=s.get('user_id'), location_id=s.get('context_id'), parent_id=destination.get('user_id'))
                res += 1
            return res

    def deleteFromRequest(self, request, removed=None, ):
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    super().filter(id=id).delete()
                    res += 1

            return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'user_id': record.user.id,
            'user__username': record.user.username,
            'user__first_name': record.user.first_name,
            'user__last_name': record.user.last_name,
            'user__email': record.user.email,
            'user__middle_name': record.user.middle_name,
            'compulsory_reading': record.props.compulsory_reading,
            'props': record.props,
            'editing': record.editing,
            'deliting': record.deliting,
            'location_id': record.location.id,
            'location_position': record.location_position,
            'parent_id': record.parent.id if record.parent else None,
        }
        return DelProps(res)

    @staticmethod
    def getRecord1(record):
        res = {
            'location_id': record.location.id,
        }
        return DelProps(res)

    def get_queryset(self):
        return Locations_usersQuerySet(self.model, using=self._db)


class Locations_users(Hierarcy):
    location = ForeignKeyCascade(Locations)
    user = ForeignKeyCascade(User)
    props = BitField(flags=(
        ('compulsory_reading', 'Обязательное прочтение'),  # 1
    ), default=1, db_index=True)

    @property
    def location_position(self):
        from kaf_pas.input.models.user_add_info import User_add_info
        res = []
        for item in User_add_info.objects.filter(user=self.user):
            res.append(f'{item.location.full_name} : {item.position.name}')
        return ', '.join(res)

    objects = Locations_usersManager()

    def __str__(self):
        return f"ID:{self.id}"

    class Meta:
        constraints = [
            UniqueConstraint(fields=['location', 'user'], condition=Q(parent=None), name='Locations_users_unique_constraint_0'),
            UniqueConstraint(fields=['location', 'parent', 'user'], name='Locations_users_unique_constraint_1'),
        ]
        verbose_name = 'Ответственные пользователи ресурсов'



import logging
from datetime import datetime

from django.conf import settings
from django.db import transaction, connection
from django.db.models import TextField, DateTimeField
from django.forms import model_to_dict

import kaf_pas
from isc_common import StackElementNotExist, delAttr, setAttr
from isc_common.auth.models.user import User
from isc_common.bit import TurnBitOn
from isc_common.common import blinkString
from isc_common.common.mat_views import create_tmp_mat_view, drop_mat_view
from isc_common.datetime import DateToStr
from isc_common.fields.code_field import CodeStrictField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet
from isc_common.progress import managed_progress, ProgressDroped
from kaf_pas.planing.models.operation_types import Operation_types
from kaf_pas.planing.models.status_operation_types import Status_operation_types
from kaf_pas.production.models import progress_deleted
from kaf_pas.production.models.launch_operation_material import Launch_operations_material
from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources
from kaf_pas.production.models.launch_operations_item import Launch_operations_item

logger = logging.getLogger(__name__)


class OperationsQuerySet(AuditQuerySet):
    def create(self, **kwargs):
        from isc_common.seq import get_deq_next_value
        if kwargs.get('num') == None:
            setAttr(kwargs, 'num', str(get_deq_next_value('planing_operations_id_seq')))
        return super().create(**kwargs)


class Route_item():

    def __init__(self, item_id, first_operation, last_operation):
        self.item_id = item_id
        self.first_operation = first_operation
        self.last_operation = last_operation

    def __str__(self):
        return f'item_id: {self.item_id}, first_operation: [{self.first_operation}], last_operation: [{self.last_operation}]'


class OperationPlanItem:
    id = None
    item = None
    resource = None
    production_operation = None
    value = None
    value1 = None

    def __init__(self, *args, **kwargs):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.production.models.resource import Resource
        from kaf_pas.planing.models.operation_value import Operation_value
        from kaf_pas.planing.models.levels import Levels

        if len(kwargs) == 0:
            raise Exception(f'{self.__class__.__name__} kwargs is empty')

        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

        if isinstance(self.item, int):
            self.item = Item.objects.get(id=self.item)

        if isinstance(self.resource, int):
            self.resource = Resource.objects.get(id=self.resource)

        if isinstance(self.production_operation, int):
            self.production_operation = kaf_pas.production.models.operations.Operations.objects.get(id=self.production_operation)

        if isinstance(self.value, int):
            self.value = Operation_value.objects.get(id=self.value)

        if isinstance(self.value1, int):
            self.value1 = Operation_value.objects.get(id=self.value1)

        if isinstance(self.operation_level, int):
            self.operation_level = Levels.objects.get(id=self.operation_level)


class OperationsManager(AuditManager):

    @staticmethod
    def delete_recursive(operation, soft_delete=False, user=None):
        from isc_common.models.deleted_progresses import Deleted_progresses
        try:
            settings.LOCKS.acquire('OperationsManager.delete_recursive')
            with transaction.atomic():
                from kaf_pas.planing.models.operation_refs import Operation_refs

                count = Operation_refs.objects.get_descendants_count(id=operation.id)
                if count > 0:
                    with managed_progress(
                            id=operation.id,
                            qty=Operation_refs.objects.get_descendants_count(id=operation.id),
                            user=user,
                            message='Удаление связанных операций операций',
                            title='Выполнено',
                            props=TurnBitOn(0, 0)
                    ) as progress:
                        for item in Operation_refs.objects.get_descendants(id=operation.id, order_by_clause='order by level desc'):
                            if not soft_delete:
                                Operation_refs.objects.filter(id=item.id).delete()
                            else:
                                Operation_refs.objects.filter(id=item.child_id).soft_delete()

                            step_res = progress.step()

                            if step_res != 0:
                                raise ProgressDroped(progress_deleted)

                if not soft_delete:
                    Operations.objects.filter(id=operation.id).delete()
                else:
                    Operations.objects.filter(id=operation.id).soft_delete()
            settings.LOCKS.release('OperationsManager.delete_recursive')
        except ProgressDroped as ex:
            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
            settings.LOCKS.release('OperationsManager.delete_recursive')
            raise ex

    def updateFromRequest(self, request, removed=None, function=None):
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.planing.models.operation_value import Operation_value
        from kaf_pas.planing.models.operation_color import Operation_color

        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'creator_id')
        _data.setdefault('creator_id', request.user_id)

        item_id = _data.get('item_id')
        edizm_id = _data.get('edizm_id')
        value = _data.get('value')
        color_id = _data.get('color_id')
        operation_id = _data.get('id')

        delAttr(_data, 'id')
        delAttr(_data, 'creator__short_name')

        delAttr(_data, 'opertype__full_name')
        delAttr(_data, 'isFolder')

        delAttr(_data, 'status__code')
        delAttr(_data, 'status__name')

        delAttr(_data, 'location__code')
        delAttr(_data, 'location__name')
        delAttr(_data, 'location__full_name')

        delAttr(_data, 'item__STMP_1_id')
        delAttr(_data, 'item__STMP_1__value_str')

        delAttr(_data, 'item__STMP_2_id')
        delAttr(_data, 'item__STMP_2__value_str')

        delAttr(_data, 'edizm__code')
        delAttr(_data, 'edizm__name')

        if item_id:
            Operation_item.objects.update_or_create(operation_id=operation_id, defaults=dict(item_id=item_id))
        else:
            Operation_item.objects.filter(operation_id=operation_id).delete()

        if edizm_id and value:
            Operation_value.objects.update_or_create(operation_id=operation_id, defaults=dict(edizm_id=edizm_id, value=value))
        else:
            Operation_value.objects.filter(operation_id=operation_id).delete()

        if color_id:
            Operation_color.objects.update_or_create(operation_id=operation_id, defaults=dict(color_id=color_id))
        else:
            Operation_color.objects.filter(operation_id=operation_id).delete()

        with transaction.atomic():
            super().update_or_create(id=operation_id, defaults=_data)
            return data

    def deleteFromRequest(self, request, removed=None, ):

        request = DSRequest(request=request)
        res = 0

        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    for operation in super().filter(id=id):
                        OperationsManager.delete_recursive(operation=operation, user=request.user)
                        operation.delete()
                        res += 1
        return res

    @staticmethod
    def make_routing(data):
        from isc_common import Stack
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.ckk.models.ed_izm import Ed_izm
        from kaf_pas.planing.models.levels import Levels
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.planing.models.operation_level import Operation_level
        from kaf_pas.planing.models.operation_material import Operation_material
        from kaf_pas.planing.models.operation_operation import Operation_operation
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_resources import Operation_resources
        from kaf_pas.planing.models.operation_standard_prod import Operation_standard_prod
        from kaf_pas.planing.models.operation_value import Operation_value
        from kaf_pas.production.models.launches import Launches

        # logger.debug(f'data: {data}')

        launch_id = data.get('id')
        launch = Launches.objects.get(id=launch_id)
        user = data.get('user')
        if not isinstance(user, User):
            raise Exception(f'user must be User instance.')
        edizm = Ed_izm.objects.get(code='шт')

        step_res = 0

        if launch.status.code == 'route_made':
            raise Exception(f'Маршрутизация уже выполнена.')

        sql_str = f'''with a as (
                                    with recursive r as (
                                        select *,
                                               1 as level
                                        from production_launch_item_refs
                                        where parent_id is null
                                           and launch_id = {launch_id}
                                           and is_bit_on(props::int,0) = true
                                        union all
                                        select production_launch_item_refs.*,
                                               r.level + 1 as level
                                        from production_launch_item_refs
                                                 join r
                                                      on
                                                          production_launch_item_refs.parent_id = r.child_id
                                        where is_bit_on(r.props::int,0) = true
                                        and is_bit_on(production_launch_item_refs.props::int,0) = true
                                    )
                                
                                    select r1.id,
                                           r1.parent_id,
                                           r1.child_id,
                                           r1.launch_id,
                                           r1.qty,
                                           r1.qty_per_one,
                                           r1.level,
                                           r1.item_full_name,
                                           r1.item_full_name_obj                 
                                    from (select distinct r.id,
                                                          r.parent_id,
                                                          r.child_id,
                                                          r.launch_id,
                                                          r.item_full_name,
                                                          r.item_full_name_obj,
                                                          r.qty,
                                                          r.qty_per_one,  
                                                          (
                                                              select plil.section
                                                              from production_launch_item_line plil
                                                              where plil.child_id = r.child_id
                                                                and plil.parent_id = r.parent_id
                                                                and plil.launch_id = r.launch_id) section,
                                                          level
                                          from r
                                                   join ckk_item ci on ci.id = r.child_id
                                          where r.launch_id = {launch_id}
                                          order by level desc) r1
                                    where lower(r1.section) != 'документация'
                                       or r1.parent_id is null
                                )
                                
                                
                                select a.id,
                                       a.parent_id,
                                       a.child_id,
                                       a.launch_id,
                                       a.qty,
                                       a.qty_per_one,
                                       a.level,
                                       a.item_full_name,
                                       a.item_full_name_obj
                                from a'''

        mat_view_name = create_tmp_mat_view(sql_str=sql_str, indexes=['parent_id', 'child_id'])
        with connection.cursor() as cursor:
            cursor.execute(f'select count(*) from {mat_view_name}')
            count, = cursor.fetchone()

        try:
            settings.LOCKS.acquire('OperationsManager.make_routing')
            with transaction.atomic():
                with managed_progress(
                        id=f'launch_{launch.id}_{user.id}',
                        qty=count * 2,
                        user=user,
                        message=f'<h3>Расчет маршрутизации внутри товарных позиций, Запуск № {launch.code} от {DateToStr(launch.date)}</h3>',
                        title='Выполнено',
                        props=TurnBitOn(0, 0)
                ) as progress:
                    progress.except_func = lambda progress: drop_mat_view(mat_view_name)

                    for launch in Launches.objects.filter(id=launch.id).select_for_update():
                        with connection.cursor() as cursor:
                            cursor.execute(f'select max(level), min(level) from {mat_view_name}')
                            rows = cursor.fetchone()
                            min_level, max_level = rows

                            cursor.execute(f'select * from {mat_view_name} order by level desc')
                            rows = cursor.fetchall()

                            routed_items = Stack()

                            for row in rows:
                                def make_oparetions(row, mode='child'):
                                    id, parent_id, child_id, launch_id, qty, qty_per_one, level, item_full_name, item_full_name_obj = row

                                    # Более низкий уровень в иерархии товарной позиции соответствует более высокому в маршрутизации, т.к. необходимо изготавливать ранньше
                                    level = max_level - (level - min_level)
                                    logger.debug(f'level: {level}')

                                    if mode == 'child':
                                        cursor.execute(f'select * from {mat_view_name} where qty is null and child_id = %s', [child_id])
                                        null_rows = cursor.fetchall()
                                        if len(null_rows) > 0:
                                            nulls_array = []
                                            for null_row in null_rows:
                                                id, parent_id, child_id, launch_id, qty, qty_per_one, level, item_full_name, item_full_name_obj = null_row
                                                nulls_str = f'<b>ID: {id}: {item_full_name}</b>'
                                                nulls_array.append(nulls_str)
                                            nulls_str = f'''{blinkString(text='Не указано количество : ', color='red')}<br><div>{'<br>'.join(nulls_array)}</div>'''
                                            raise Exception(nulls_str)

                                        cursor.execute(f'select sum(qty) from {mat_view_name} where child_id = %s', [child_id])
                                        qty, = cursor.fetchone()
                                        logger.debug(f'qty: {qty}')

                                    elif mode == 'parent':
                                        if parent_id != None:
                                            child_id = parent_id

                                    if len(routed_items.find(lambda child_item: child_item.item_id == child_id)) == 0:
                                        income_operation = None
                                        first_operation = None

                                        # Выполняем маршрутизацию внутри товарной позиции согласно порядку выплонения оперций из production
                                        cnt1 = Launch_operations_item.objects.filter(item_id=child_id, launch_id=launch_id).count()
                                        if cnt1 > 0:
                                            for launch_operations_item in Launch_operations_item.objects.filter(item_id=child_id, launch_id=launch_id).order_by('num'):

                                                outcome_operation = Operations.objects.create(
                                                    date=datetime.now(),
                                                    opertype=settings.OPERS_TYPES_STACK.ROUTING_TASK,
                                                    status=settings.OPERS_TYPES_STACK.ROUTING_TASK_STATUSES.get('new'),
                                                    creator=user
                                                )
                                                logger.debug(f'Created outcome_operation :  {outcome_operation}')

                                                operation_launches = Operation_launches.objects.create(operation=outcome_operation, launch=launch)
                                                logger.debug(f'Created operation_launches :  {operation_launches}')

                                                operation_item = Operation_item.objects.create(
                                                    operation=outcome_operation,
                                                    item=launch_operations_item.item,
                                                    item_full_name=item_full_name,
                                                    item_full_name_obj=item_full_name_obj
                                                )
                                                logger.debug(f'Created operation_item :  {operation_item}')

                                                operation_operation = Operation_operation.objects.create(
                                                    operation=outcome_operation,
                                                    production_operation=launch_operations_item.operation,
                                                    num=launch_operations_item.num,
                                                    qty=launch_operations_item.qty,
                                                    ed_izm=launch_operations_item.ed_izm,
                                                )
                                                logger.debug(f'Created operation_operation :  {operation_operation}')

                                                _level, created = Levels.objects.get_or_create(
                                                    code=str(level),
                                                    defaults=dict(
                                                        name=str(level),
                                                        editing=False,
                                                        deliting=False
                                                    ))
                                                if created:
                                                    logger.debug(f'Created level :  {_level}')

                                                operation_level = Operation_level.objects.create(operation=outcome_operation, level=_level)
                                                logger.debug(f'Created operation_level :  {operation_level}')

                                                if qty != None:
                                                    operation_value = Operation_value.objects.create(operation=outcome_operation, value=qty_per_one, edizm=edizm, props=Operation_value.props.perone)
                                                    logger.debug(f'Created operation_value :  {operation_value}')
                                                    operation_value = Operation_value.objects.create(operation=outcome_operation, value=qty, edizm=edizm)
                                                    logger.debug(f'Created operation_value :  {operation_value}')

                                                for launch_operation_material in Launch_operations_material.objects.filter(launch_operationitem=launch_operations_item):
                                                    if launch_operation_material.material_askon != None:
                                                        operation_material = Operation_material.objects.create(operation=outcome_operation, material=launch_operation_material.material_askon)
                                                        logger.debug(f'Created operation_material :  {operation_material}')

                                                    if launch_operation_material.material != None:
                                                        operation_standard_prod = Operation_standard_prod.objects.get_or_create(operation=outcome_operation, material=launch_operation_material.material)
                                                        logger.debug(f'Created operation_standard_prod :  {operation_standard_prod}')

                                                def exception_not_resource():
                                                    from isc_common.common import blinkString
                                                    raise Exception(f'''<b>Для : {item_full_name}</b>  {blinkString(text='не задан ресурс.  Запустите анализатор готовности к запуску.', blink=False, color='red', bold=True)}''')

                                                if Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item) == 0:
                                                    exception_not_resource()

                                                for launch_operation_resources in Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item):
                                                    operation_resources = Operation_resources.objects.create(operation=outcome_operation, resource=launch_operation_resources.resource)
                                                    logger.debug(f'Created operation_resources :  {operation_resources}')

                                                if income_operation == None:
                                                    first_operation = outcome_operation

                                                operation_refs = Operation_refs.objects.create(
                                                    parent=income_operation,
                                                    parent_real=income_operation,
                                                    child=outcome_operation,
                                                    props=Operation_refs.props.inner_routing
                                                )
                                                logger.debug(f'Created operation_refs :  {operation_refs}')

                                                income_operation = outcome_operation
                                                cnt1 -= 1
                                                if cnt1 == 0:
                                                    routed_items.push(Route_item(item_id=child_id, first_operation=first_operation, last_operation=outcome_operation), logger=logger)
                                        else:
                                            def exception_not_operations():
                                                from isc_common.common import blinkString
                                                raise Exception(f'''<b>Для : {item_full_name}</b>  {blinkString(text='не задано ни одной операции. Запустите анализатор готовности к запуску.', blink=False, color='red', bold=True)}''')

                                            exception_not_operations()
                                    # else:
                                    #     raise Exception(f'child_id exisys yet !!!!')

                                make_oparetions(row=row)

                                step_res = progress.step()
                                if step_res != 0:
                                    raise ProgressDroped(progress_deleted)

                            if step_res == 0:
                                # Выполняем маршрутизацию между товарными позициями соединяя последнюю оперцию предыдущей товарной позиции с первой операциеей следующей
                                # товарной позиции
                                progress.setContentsLabel(f'<h3>Расчет маршрутизации между товарными позициями, Запуск № {launch.code} от {DateToStr(launch.date)}</h3>')

                                for row in rows:
                                    id, parent_id, child_id, launch_id, qty, qty_per_one, level, item_full_name, item_full_name_obj = row
                                    try:
                                        if parent_id == None:
                                            parent_id = child_id
                                        parent_item = routed_items.find_one(lambda item: item.item_id == parent_id)
                                    except StackElementNotExist:
                                        logger.warning(f'parent_id: {parent_id} !!!!!!!!!!!!!!!!!!Товарная позиция не обнаружена среди товарных позиций, прошедших внутреннюю маршрутизацию !!!!!!!!!!!!!!!!')
                                        # Если товарная позиция не обнаружена среди товарных позиций, прошедших внутреннюю маршрутизацию

                                        make_oparetions(row=row, mode='parent')
                                        parent_item = routed_items.find_one(lambda item: item.item_id == parent_id)

                                    cursor.execute(f'''select child_id from {mat_view_name} where parent_id = %s''', [parent_id])
                                    parents_rows = cursor.fetchall()
                                    for parents_row in parents_rows:
                                        _child_id, = parents_row
                                        _child = routed_items.find_one(lambda item: item.item_id == _child_id)

                                        operation_refs, created = Operation_refs.objects.get_or_create(
                                            parent=_child.last_operation,
                                            parent_real=_child.last_operation,
                                            child=parent_item.first_operation,
                                            defaults=dict(
                                                props=Operation_refs.props.outer_routing
                                            )
                                        )
                                        logger.debug(f'Created operation_refs :  {operation_refs}')

                                        deleted, _ = Operation_refs.objects.filter(parent__isnull=True, child=parent_item.last_operation).delete()
                                    step_res = progress.step()
                                    if step_res != 0:
                                        raise ProgressDroped(progress_deleted)

                            if step_res == 0:
                                launch.status = settings.PROD_OPERS_STACK.ROUTMADE
                                launch.save()

                                progress.sendMessage(type='refresh_launches_grid')
                                settings.EVENT_STACK.EVENTS_PRODUCTION_MAKE_ROUTING.send_message(f'<h3>Выполнен Расчет маршрутизации: Запуск № {launch.code} от {DateToStr(launch.date)}</h3><p/>')

                    drop_mat_view(mat_view_name)

                    if step_res != 0:
                        raise ProgressDroped(progress_deleted)
            settings.LOCKS.release('OperationsManager.make_routing')
        except ProgressDroped as ex:
            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
            settings.LOCKS.release('OperationsManager.make_routing')
            raise ex

    @staticmethod
    def clean_routing(data):
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.production.models.launches import Launches

        launch_id = data.get('id')
        launch = Launches.objects.get(id=launch_id)

        if launch.status.code == 'formirovanie':
            raise Exception(f'Маршрутизация уже удалена.')

        # logger.debug(f'data: {data}')

        launch_id = data.get('id')
        launch = Launches.objects.get(id=launch_id)

        user = data.get('user')
        settings.LOCKS.acquire('OperationsManager.clean_routing')
        query = Operation_launches.objects.filter(launch=launch, operation__opertype__in=[
            settings.OPERS_TYPES_STACK.ROUTING_TASK,
        ])
        step_res = 0

        with managed_progress(
                id=launch.id,
                qty=query.count(),
                user=user,
                message=f'<h3>Удаление маршрутизации: Запуск № {launch.code} от {DateToStr(launch.date)}</h3>',
                title='Выполнено',
                props=TurnBitOn(0, 0)
        ) as progress:
            try:
                with transaction.atomic():
                    for launch in Launches.objects.filter(id=launch.id).select_for_update():
                        launch.status = launch.status = settings.PROD_OPERS_STACK.FORMIROVANIE
                        launch.save()

                        cnt = 0
                        for operation_launches in query:
                            OperationsManager.delete_recursive(operation=operation_launches.operation, user=user)
                            step_res = progress.step()
                            cnt += 1
                            if step_res != 0:
                                raise ProgressDroped(progress_deleted)

                        if step_res == 0:
                            if cnt > 0:
                                settings.EVENT_STACK.EVENTS_PRODUCTION_DELETE_ROUTING.send_message(f'<h3>Выполнено Удаление маршрутизации: Запуск № {launch.code} от {DateToStr(launch.date)}</h3><p/>')
                            progress.sendMessage(type='refresh_launches_grid')

                    if step_res != 0:
                        raise ProgressDroped(progress_deleted)
            except ProgressDroped as ex:
                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                raise ex
        settings.LOCKS.release('OperationsManager.clean_routing')

        return model_to_dict(launch)

    @staticmethod
    def get_locations_users_query(resource, item_full_name):
        from kaf_pas.ckk.models.locations_users import Locations_users

        locations_users_query = Locations_users.objects.filter(location=resource.location, parent__isnull=True)
        if locations_users_query.count() == 0:
            raise Exception(blinkString(text=f'Не обнаружен ответственный исполнитель для : {resource.location.full_name}, \n{item_full_name}', bold=True))

        return locations_users_query

    @staticmethod
    def get_resource_workshop(resource):
        from kaf_pas.ckk.models.locations import Locations

        res = None
        for location in Locations.objects_tree.get_parents(id=resource.location.id, child_id='id', include_self=False):
            if location.props.isWorkshop == True:
                res, _ = settings.OPERS_STACK.NOT_UNDEFINED_WORKSHOP(location)
                return res

        if res == None:
            raise Exception(f'Не обнаружен цех, с признаком "Уровень цеха" для : Location ID: {resource.location.id} {resource.location.full_name}, Resource ID: {resource.id}: {resource.name}')
        return res

    @staticmethod
    def make_production_order(data):
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.planing.models.operation_executor import Operation_executor
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.planing.models.operation_level import Operation_level
        from kaf_pas.planing.models.operation_operation import Operation_operation
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_resources import Operation_resources
        from kaf_pas.planing.models.operation_value import Operation_value
        from kaf_pas.planing.models.operations_view import Operations_view
        from kaf_pas.production.models.launches import Launches
        from kaf_pas.production.models.launches import Launches
        from kaf_pas.production.models.resource import Resource

        user = data.get('user')
        if isinstance(user, int):
            user = User.objects.get(id=user)

        launch = data.get('id')
        if isinstance(launch, int):
            launch = Launches.objects.get(id=launch)

        if launch.status.code == 'in_production':
            raise Exception(f'Формирование заданий на производство уже выполнено.')

        item_query = Operations_view.objects.filter(
            launch=launch,
            production_operation__isnull=False,
            opertype=settings.OPERS_TYPES_STACK.ROUTING_TASK
        ).values('item', 'item_full_name', 'operation_level').distinct()
        qty = item_query.count()
        logger.debug(f'Calculated qty : {qty}')

        new_status_order_prod = settings.OPERS_TYPES_STACK.PRODUCTION_TASK_STATUSES.get('new')
        new_status_det_order_prod = settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK_STATUSES.get('new')
        operation_executors = set()

        settings.LOCKS.acquire('OperationsManager.make_production_order')
        with managed_progress(
                id=f'order_by_prod_launch_{launch.id}_{user.id}',
                qty=item_query.count(),
                user=user,
                message=f'<h3>Расчет заданий на производство, Запуск № {launch.code} от {DateToStr(launch.date)}</h3>',
                title='Выполнено',
                props=TurnBitOn(0, 0)
        ) as progress:
            try:
                with transaction.atomic():
                    for launch in Launches.objects.filter(id=launch.id).select_for_update():
                        for route_oparation_item in item_query:
                            route_oparation_item = OperationPlanItem(**route_oparation_item)

                            # Головная операция заказа
                            main_oper_production_order = Operations.objects.create(
                                date=datetime.now(),
                                opertype=settings.OPERS_TYPES_STACK.PRODUCTION_TASK,
                                status=new_status_order_prod,
                                creator=user,
                                editing=False,
                                deliting=False
                            )
                            logger.debug(f'Created operation :  {main_oper_production_order}')

                            operation_launches = Operation_launches.objects.create(operation=main_oper_production_order, launch=launch)
                            logger.debug(f'Created operation_launches :  {operation_launches}')

                            operation_item = Operation_item.objects.create(
                                operation=main_oper_production_order,
                                item=route_oparation_item.item,
                                item_full_name=route_oparation_item.item_full_name
                            )
                            logger.debug(f'Created operation_item :  {operation_item}')

                            operation_refs = Operation_refs.objects.create(
                                parent_real_id=route_oparation_item.id,
                                child=main_oper_production_order,
                                props=Operation_refs.props.product_order_routing
                            )
                            logger.debug(f'Created operation_refs :  {operation_refs}')

                            route_items_operations_query = Operations_view.objects.filter(
                                launch=launch,
                                item=route_oparation_item.item,
                                production_operation__isnull=False,
                                opertype=settings.OPERS_TYPES_STACK.ROUTING_TASK).values(
                                'id',
                                'resource',
                                'production_operation',
                                'value',
                                'value1',
                                'operation_level',
                                'production_operation_num',
                                'production_operation_qty',
                                'production_operation_edizm',
                                'item_full_name',
                            ).order_by('production_operation_num').distinct()

                            qty = route_items_operations_query.count()
                            logger.debug(f'Calculated qty : {qty}')
                            top_resource = None

                            for route_items_operations in route_items_operations_query:
                                route_items_operations = OperationPlanItem(**route_items_operations)

                                production_operation = kaf_pas.production.models.operations.Operations.objects.get(id=route_items_operations.production_operation.id)

                                # Пытаемся распознать операцию комплектации
                                # if production_operation.props.assemly == True:
                                #     raise Exception(f'Meeting production_operation :  {production_operation} is made_common_form operation')

                                # Операция детализации заказа
                                detail_oper_production_order = Operations.objects.create(
                                    date=datetime.now(),
                                    opertype=settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK,
                                    status=new_status_det_order_prod,
                                    creator=user,
                                    editing=False,
                                    deliting=False
                                )
                                logger.debug(f'Created order_prod_item_det_operation :  {detail_oper_production_order}')

                                operation_launches = Operation_launches.objects.create(operation=detail_oper_production_order, launch=launch)
                                logger.debug(f'Created operation_launches :  {operation_launches}')

                                operation_item = Operation_item.objects.create(
                                    operation=detail_oper_production_order,
                                    item=route_oparation_item.item,
                                    item_full_name=route_oparation_item.item_full_name
                                )
                                logger.debug(f'Created operation_item :  {operation_item}')

                                operation_level = Operation_level.objects.create(
                                    operation=detail_oper_production_order,
                                    level=route_items_operations.operation_level,
                                )
                                logger.debug(f'Created operation_level :  {operation_level}')

                                resource = Resource.objects.get(id=route_items_operations.resource.id)
                                if top_resource == None:
                                    top_resource = OperationsManager.get_resource_workshop(resource)
                                    operation_resources = Operation_resources.objects.create(
                                        operation=main_oper_production_order,
                                        resource=top_resource
                                    )
                                    logger.debug(f'Created operation_resources :  {operation_resources}')

                                    operation_value = Operation_value.objects.create(
                                        operation=main_oper_production_order,
                                        value=route_items_operations.value.value,
                                        edizm=route_items_operations.value.edizm
                                    )
                                    logger.debug(f'Created operation_value :  {operation_value}')

                                    operation_value = Operation_value.objects.create(
                                        operation=main_oper_production_order,
                                        value=route_items_operations.value1.value,
                                        edizm=route_items_operations.value1.edizm,
                                        props=Operation_value.props.perone
                                    )
                                    logger.debug(f'Created operation_value :  {operation_value}')

                                    operation_level = Operation_level.objects.create(
                                        operation=main_oper_production_order,
                                        level=route_items_operations.operation_level,
                                    )
                                    logger.debug(f'Created operation_level :  {operation_level}')

                                    locations_users_query = OperationsManager.get_locations_users_query(
                                        resource=top_resource,
                                        item_full_name=route_oparation_item.item_full_name
                                    )

                                    for locations_users in locations_users_query:
                                        operation_executor = Operation_executor.objects.create(
                                            operation=main_oper_production_order,
                                            executor=locations_users.user
                                        )
                                        logger.debug(f'Created operation_executor :  {operation_executor}')
                                        operation_executors.add(locations_users.user)

                                operation_resources = Operation_resources.objects.create(
                                    operation=detail_oper_production_order,
                                    resource=resource
                                )
                                logger.debug(f'Created operation_resources :  {operation_resources}')

                                operation_operation = Operation_operation.objects.create(
                                    operation=detail_oper_production_order,
                                    production_operation=route_items_operations.production_operation,
                                    num=route_items_operations.production_operation_num,
                                    qty=route_items_operations.production_operation_qty,
                                    ed_izm_id=route_items_operations.production_operation_edizm,
                                )
                                logger.debug(f'Created operation_operation :  {operation_operation}')

                                operation_value = Operation_value.objects.create(
                                    operation=detail_oper_production_order,
                                    value=route_items_operations.value.value,
                                    edizm=route_items_operations.value.edizm
                                )
                                logger.debug(f'Created operation_value :  {operation_value}')

                                operation_value = Operation_value.objects.create(
                                    operation=detail_oper_production_order,
                                    value=route_items_operations.value1.value,
                                    edizm=route_items_operations.value1.edizm,
                                    props=Operation_value.props.perone
                                )
                                logger.debug(f'Created operation_value :  {operation_value}')

                                # Привязка к оперции тов позиции из маршрутизации
                                operation_refs = Operation_refs.objects.create(
                                    parent_id=route_items_operations.id,
                                    parent_real_id=route_items_operations.id,
                                    child=detail_oper_production_order,
                                    props=Operation_refs.props.product_order_routing
                                )
                                logger.debug(f'Created operation_refs :  {operation_refs}')

                                # Привязка к головной операции заказа на производство
                                operation_refs = Operation_refs.objects.create(
                                    parent=main_oper_production_order,
                                    child=detail_oper_production_order,
                                    props=Operation_refs.props.product_order_routing
                                )
                                logger.debug(f'Created operation_refs :  {operation_refs}')

                            step_res = progress.step()
                            if step_res != 0:
                                raise ProgressDroped(progress_deleted)

                    launch.status = settings.PROD_OPERS_STACK.IN_PRODUCTION
                    launch.save()

                    settings.EVENT_STACK.EVENTS_PRODUCTION_ORDER_CREATE.send_message(message=f'<h3>Размешен(ы) новый(е) заказ(ы) на производство.</h3><p/>', users_array=list(operation_executors))
                    progress.sendMessage(type='refresh_launches_grid')

            except ProgressDroped as ex:
                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                raise ex
        settings.LOCKS.release('OperationsManager.make_production_order')

    @staticmethod
    def delete_production_order(data):
        from kaf_pas.production.models.launches import Launches
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from isc_common.models.deleted_progresses import Deleted_progresses

        user = data.get('user')
        if isinstance(user, int):
            user = User.objects.get(id=user)

        launch = data.get('id')
        if isinstance(launch, int):
            launch = Launches.objects.get(id=launch)

        if launch.status.code == 'route_made':
            raise Exception(f'Удалений заданий на производство уже выполнено.')

        opertypes = (settings.OPERS_TYPES_STACK.PRODUCTION_TASK.id, settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK.id)
        settings.LOCKS.acquire('OperationsManager.delete_production_order')
        with connection.cursor() as cursor:
            cursor.execute('''WITH RECURSIVE r AS (
                                                SELECT *, 1 AS level
                                                FROM planing_operations_view
                                                WHERE opertype_id in %s
                                                AND launch_id=%s
                                                AND is_bit_on(props::integer, 2) = true

                                                union all

                                                SELECT planing_operations_view.*, r.level + 1 AS level
                                                FROM planing_operations_view
                                                         JOIN r
                                                              ON planing_operations_view.parent_id = r.id
                                                WHERE planing_operations_view.launch_id=%s
                                                AND r.launch_id=%s              
                                                AND is_bit_on(r.props::integer, 2) = true
                                                AND is_bit_on(planing_operations_view.props::integer, 2) = true
                                                )
                                                              

                                            select distinct a.*
                                            from (
                                                     select distinct id, level
                                                     from r
                                                     where opertype_id in %s
                                                 ) as a
                                            order by a.level desc''', [opertypes, launch.id, launch.id, launch.id, opertypes])
            rows = cursor.fetchall()

            qty = len(rows)
            logger.debug(f'Calculated qty : {qty}')
            with managed_progress(
                    id=f'delete_order_by_prod_launch_{launch.id}_{user.id}',
                    qty=qty,
                    user=user,
                    message=f'<h3>Удаление заданий на производство, Запуск № {launch.code} от {DateToStr(launch.date)}</h3>',
                    title='Выполнено',
                    props=TurnBitOn(0, 0)
            ) as progress:
                try:
                    with transaction.atomic():
                        for launch in Launches.objects.filter(id=launch.id).select_for_update():
                            for row in rows:
                                operation_id, _ = row
                                Operation_refs.objects.filter(child_id=operation_id).delete()
                                Operations.objects.filter(id=operation_id).delete()
                                if progress.step() != 0:
                                    raise ProgressDroped(progress_deleted)

                        launch.status = settings.PROD_OPERS_STACK.ROUTMADE
                        launch.save()
                    progress.sendMessage(type='refresh_launches_grid')
                except ProgressDroped as ex:
                    Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                    raise ex
        settings.LOCKS.release('OperationsManager.delete_production_order')

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'num': record.num,
            'date': record.date,
            'opertype_id': record.opertype.id,
            'opertype__full_name': record.opertype.full_name,
            'status_id': record.status.id if record.status else None,
            'status__code': record.status.code if record.status else None,
            'status__name': record.status.name if record.status else None,
            'description': record.description,
        }
        return res

    def get_queryset(self):
        return OperationsQuerySet(self.model, using=self._db)


class Operations(AuditModel):
    num = CodeStrictField(blank=True, null=True)
    date = DateTimeField()
    creator = ForeignKeyProtect(User)
    opertype = ForeignKeyProtect(Operation_types)
    status = ForeignKeyProtect(Status_operation_types, related_name='planing_Operations_status')
    prev_status = ForeignKeyProtect(Status_operation_types, related_name='planing_Operations_prev_status', null=True, blank=True)
    description = TextField(null=True, blank=True)

    @property
    def value_start(self):
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_value import Operation_value
        try:
            # LAUNCH_TSK - /Задание на производство/Запуск задания на производство
            res = Operation_refs.objects.get(parent_id=self.id, child__opertype__code='LAUNCH_TSK')
            return Operation_value.objects.get(operation=res.child).value
        except Operation_refs.DoesNotExist:
            return 0
        except Operation_refs.MultipleObjectsReturned:
            Operation_refs.objects.filter(parent_id=self.id, child__opertype__code='LAUNCH_TSK').delete()
            return 0

    objects = OperationsManager()

    def __str__(self):
        return f"ID:{self.id}, date: {self.date}, description: {self.description},  creator: [{self.creator}], opertype: [{self.opertype}], prev_status: [{self.prev_status}], status: [{self.status}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Опреации системные'



# Generated by Django 3.0.3 on 2020-02-11 09:02

from django.db import migrations, models
import django.db.models.deletion
import django.db.models.expressions
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0138_auto_20200211_0812'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lotsman_document_attr_cross',
            name='parent_document',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='Lotsman_document_attr_cross_parent_doc', to='kd.Lotsman_documents_hierarcy'),
        ),
        migrations.AlterUniqueTogether(
            name='lotsman_document_attr_cross',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='lotsman_document_attr_cross',
            constraint=models.CheckConstraint(check=models.Q(_negated=True, document=django.db.models.expressions.F('parent_document')), name='Lotsman_document_attr_cross_not_circulate_refs'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_document_attr_cross',
            constraint=models.UniqueConstraint(condition=models.Q(parent_document=None), fields=('attribute', 'document'), name='Lotsman_document_attr_cross_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_document_attr_cross',
            constraint=models.UniqueConstraint(fields=('attribute', 'document', 'parent_document'), name='Lotsman_document_attr_cross_unique_constraint_1'),
        ),
    ]



import logging

from django.core.management import BaseCommand

from kaf_pas.planing.models.operation_operation import Operation_operation
from kaf_pas.planing.models.operation_refs import Operation_refs
from kaf_pas.planing.models.operations_view import Operations_view

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        i = 1
        # for operation in Operations_view.objects.filter(item_id=3379400, launch_id=30, operation_level_id=9).distinct():
        #     print(f'\n======Prod ({i})======')
        #     print(f'ID: {operation.id}, PARENT_ID: {operation.parent_id}')
        #     if operation.parent_id:
        #         print(Operation_operation.objects.get(operation_id=operation.parent_id).production_operation)
        #     print(operation.operation_level)
        #     print(operation.production_operation)
        #     print(operation.resource)
        #     print(operation.edizm)
        #     print(operation.value)
        #     i += 1
        #     print('======End Prod======\n')

        for operation in Operation_refs.objects.get_descendants(id=26745, order_by_clause='order by level'):
            print(f'\n======Prod ({i})======')
            print(operation)
            # print(f'ID: {operation.id}, PARENT_ID: {operation.parent_id}')
            # if operation.parent_id:
            #     print(f'parent production_operation: {Operation_operation.objects.get(operation_id=operation.parent_id).production_operation}')
            # print(f'operation_level: {operation.operation_level}')
            # print(f'production_operation: {operation.production_operation}')
            # print(f'resource: {operation.resource}')
            # print(f'edizm: {operation.edizm}')
            # print(f'value: {operation.value}')
            i += 1
            print('======End Prod======\n')

        # for operation in Operation_refs.objects.get_parents(id=23219, order_by_clause='order by level desc'):
        #     for prod_operation in Operation_operation.objects.filter(operation=operation.child):
        #         print('\n======Prod======')
        #         print(prod_operation)
        #         print('======End Prod======\n')
        #     print(operation)

        # for operation in Operations_view.objects.filter(id=23219):
        #     print(operation)
